function data = f_data_load_sa(name_group,options)
%
% data loader function with seasonal adjustment and outlier correction
%
% C Schumacher, 25 Jan 2007
%

% options
data_file = options.name_file;
freq = options.freq;
name_sheet = options.name_sheet;
data_rows = options.data_rows;
group_rows = options.group_rows;
data_columns = options.data_columns;
names_row = options.names_row;
trafo_rows = options.trafo_rows;
dates = options.dates;
src_path = options.src_path;
dum_print = options.flag_print;
ta_flag = options.ta_flag;

% load group name
data.name_group = name_group;
if (dum_print ~= 0)
    disp(' '); 
    disp(['load names and data on ', data.name_group, ', file: ' data_file, ...
        ', sheet: ', name_sheet, ', area: ', [ deblank(data_columns(1,:)), num2str(names_row), ':', deblank(data_columns(2,:)), num2str(names_row) ] ]);
end

% load series names
[data_mat_dum,name_mat] = xlsread(data_file,name_sheet, ...
    [ deblank(data_columns(1,:)), num2str(names_row), ':', deblank(data_columns(2,:)), num2str(names_row)]); % 'a2:h2'
data.name_ser = deblank(name_mat);
if (dum_print ~= 0)
    disp(' ');
    disp(['data on ', data.name_group, ', series names: ']);
    disp(data.name_ser);
end

% load group names
[data_mat_dum,name_mat] = xlsread(data_file,name_sheet, ...
    [ deblank(data_columns(1,:)), num2str(group_rows(1)), ':', deblank(data_columns(2,:)), num2str(group_rows(2))]); 
data.name_ser_group = deblank(name_mat);

% load raw data
data_mat = xlsread(data_file,name_sheet, ...
    [ deblank(data_columns(1,:)), num2str(data_rows(1,1)), ':', deblank(data_columns(2,:)), num2str(data_rows(1,2)) ]);
diff = size(dates,1)-size(data_mat,1);
data.data_raw = [data_mat;nan(diff,size(data_mat,2))];
if (dum_print ~= 0)
    disp(' ');
    disp(['data on ', data.name_group, ', raw data: ']);
    disp(data.data_raw);
end

% load transformation flags
trafo_mat = xlsread(data_file,name_sheet, ...
    [ deblank(data_columns(1,:)), num2str(trafo_rows(1,1)), ':', deblank(data_columns(2,:)), num2str(trafo_rows(1,2)) ]);
data.trafo_mat = trafo_mat;

% time aggregation
if (ta_flag ==1)
    % m data
    data_m.name_ser = data.name_ser;
    data_m.data = data.data_raw;
    data_m.dates = dates;
    data_m.stock_flow = (2.0).*ones(1,size(data_m.data,2)); % I(0) flow
    % setup quarterly calendar
    start_year  = 1959;
    start_quart = 1;
    end_year    = 2008;
    end_quart   = 4; 
    period = 1;
    cal = [start_year,start_quart,period];
    i_year  = start_year;
    i_quart = start_quart;
    while ((i_year ~= end_year) || (i_quart ~= end_quart))
        if (i_quart == 4)
            i_quart = 1;
            i_year = i_year+1;
        else
            i_quart = i_quart+1;
        end
        period = period+1;
        cal = [ cal; [i_year,i_quart,period] ];
    end
    data_q.dates = cal(:,1:2);
    data_q.data = nan(size(data_q.dates,1),1);
    %
    data_mq = f_time_aggregate_data(data_m,data_q);
    dates = data_mq.dates;
    data.dates = data_mq.dates;
    data.data_raw = data_mq.data; 
end

% make stationary
if (dum_print ~= 0)
    disp(' ');
    disp(['ensure stationarity of group ', data.name_group, ': ']);
end
for i = 1:size(data.trafo_mat,2)
    row_trafo = 2;
    index_notnan = find(not(isnan(data.data_raw(:,i)))); 
    ser = data.data_raw(index_notnan,i);
    ser_trafo = f_makestat(ser,data.trafo_mat(row_trafo,i));
    data.data_trafo(:,i) = data.data_raw(:,i);
    data.data_trafo(index_notnan,i) = ser_trafo;
    if (dum_print ~= 0)
        disp([' series ', data.name_ser{i}, ' made stationary, transformation index ', num2str(data.trafo_mat(row_trafo,i))]);
    end
end

% outlier correction by checking interquartile range
if (dum_print ~= 0)
    disp(' ');
    disp(['correct outliers for group ', data.name_group, ': ']);
end
for i = 1:size(data.trafo_mat,2)
    row_outlier = 3;
    if ( data.trafo_mat(row_outlier,i) == 1 )
        index_notnan = find(not(isnan(data.data_raw(:,i)))); 
        ser = data.data_trafo(index_notnan,i);
        med = median(ser,1);
        pct75 = prctile(ser,75.0);
        pct25 = prctile(ser,25.0);
        z = 6;
        upper_bound = med+abs(pct75-med)*z;
        lower_bound = med-abs(pct25-med)*z;
        ser_oc  = ser;
        ser_med = NaN(size(ser));
        med_t = 5;
        for t_temp=med_t+1:size(ser,1)
            ser_med(t_temp,1) = median(ser(t_temp-med_t:t_temp-1,1),1);
        end
        index = find(ser > upper_bound); ser_oc(index,:) = ser_med(index,:);
        index = find(ser < lower_bound); ser_oc(index,:) = ser_med(index,:);
        data.data_trafo_oc(:,i) = data.data_trafo(:,i);
        data.data_trafo_oc(index_notnan,i) = ser_oc;
        if (dum_print ~= 0)
            disp([' series ', data.name_ser{i}, ' oulier-corrected']);
        end
    else
        data.data_trafo_oc(:,i) = data.data_trafo(:,i);
        if (dum_print ~= 0)
            disp([' series ', data.name_ser{i}, ' not oulier-corrected']);
        end
    end
end

% use series finally?
if (dum_print ~= 0)
    disp(' ');
    disp(['remove series from group ', data.name_group, ': ']);
end
row_remove = 4;
data_full = data;
data.data_full = data_full;
index_in = find(data_full.trafo_mat(row_remove,:) ~= 0);
    data.name_ser = data_full.name_ser(:,index_in);
    data.trafo_mat = data_full.trafo_mat(:,index_in);
    data.data_raw = data_full.data_raw(:,index_in);
    data.data_trafo = data_full.data_trafo(:,index_in);
    data.data_trafo_oc = data_full.data_trafo_oc(:,index_in);
    data.name_ser_group = data_full.name_ser_group(:,index_in);
if (dum_print ~= 0)
    disp(' ');
    disp([' series kept:']);
    disp(data_full.name_ser(:,index_in));
    index_out = find(data_full.trafo_mat(row_remove,:) == 0);
    disp([' series removed:']);
    disp(data_full.name_ser(:,index_out));
end

% summary
if (dum_print ~= 0)
    disp(' ');
    disp(['group ', data.name_group, ': ']);
    disp([' no series  '  num2str(size(data.data_raw,2))]);
    disp([' no obs     '  num2str(size(data.data_raw,1))]);
    disp(' ');
end
