function [lamstarmc_out,fstarmc_out] = f_colchange_fbased(lamstarmc,fstarmc,r_fix)

[N,r,nrep]=size(lamstarmc);
T=size(fstarmc,1);
if r_fix~=r
    disp(['r_max ',num2str(r),', r_fix ',num2str(r_fix)]);
end

% fstarmc_stack=reshape(fstarmc,T,r*nrep);
legend_str_vec=cell(1,r);
for r_index=1:r
    legend_str_vec(1,r_index)={['row ',num2str(r_index)]};
end

% reshape draws into a 2-dim matrix
lamstarmc_stack=reshape(lamstarmc,N,r*nrep);
fstarmc_stack=reshape(fstarmc,T,r*nrep);

% index nan
index_notnan=not(isnan(lamstarmc_stack(1,:)));
nrep_notnan=sum(index_notnan);

% remove nans
lamstarmc_stack_notnan=lamstarmc_stack(:,index_notnan);
fstarmc_stack_notnan=fstarmc_stack(:,index_notnan);

% kmedoids clustering based on absolute pearson correlation: removes col change
disp('  kmedoids clustering based on distance defined as one minus absolute pearson correlation');
flag_method='new';
if strcmp(flag_method,'old')
    clusteroptions = struct( ...
      'numTrials', 10, ...
      'useParallel', false, ...
      'displayTrialProgress', false, ...
      'normPower', 1000, ...
      'distanceMat', [] ...
      );
    tstart = tic;
    clustStruct=f_kmedoids(fstarmc_stack_notnan',r_fix,clusteroptions); % use lamstarmc_stack' if clustering shall be based on lam
                                                                 % use r_fix clusters if more than r patterns are there
    telapsed = toc(tstart);
    disp(['  time used by kmedoids ',num2str(telapsed/(60*60)),' hrs']);
    index_kmedoids=clustStruct.clustInds;
    index_kmedoidsr=reshape(clustStruct.clustInds',r,nrep_notnan); % index_kmedoidsr contains indices 1,...,r_fix to allocate pattern from 1,...,r
elseif strcmp(flag_method,'new')
    tstart = tic;
    distfunabscorr=@(XI,XJ)(1.0-abs(corr(XI',XJ')));
    [index_kmedoids,~,avedist]=kmedoids(fstarmc_stack_notnan',r_fix,'Distance',@(Xi,Xj) distfunabscorr(Xi,Xj)); 
    telapsed = toc(tstart);
    disp(['  time used by kmedoids ',num2str(telapsed/(60*60)),' hrs']);
    index_kmedoidsr=nan(r*nrep,1);
    index_kmedoidsr(index_notnan,1)=index_kmedoids; % index_kmedoidsr contains indices 1,...,r_fix to allocate pattern from 1,...,r
    index_kmedoidsr_restack=reshape(index_kmedoidsr,r,nrep); % index_kmedoidsr contains indices 1,...,r_fix to allocate pattern from 1,...,r
    nfrep=nan(r_fix,1);
    for i=1:r_fix
        nfrep(i,1)=sum(index_kmedoidsr==i);
    end
    disp(['  no nans in index_kmedoidsr ',num2str(sum(isnan(index_kmedoidsr)))])
    disp('  how often were values in columns rearranged:');
    disp([[1:r_fix]',avedist,nfrep]);
end

% index to throw out non-unique permutations (build index, throw out later)
index_notnan_clustprop=zeros(1,nrep);
for i=1:nrep
    index_kmedoidsri=index_kmedoidsr_restack(:,i);
    index_kmedoidsri_notnan=index_kmedoidsri(not(isnan(index_kmedoidsri)));
    if length(unique(index_kmedoidsri_notnan))==length(index_kmedoidsri_notnan); 
        index_notnan_clustprop(1,i)=1;
    end
end
index_notnan_clustprop=logical(index_notnan_clustprop);
disp(['  no of unique draws ',num2str(sum(index_notnan_clustprop)),' (from ',num2str(nrep),')']); %index_kmedoidsr_restack(:,not(index_notnan_clustprop))
if sum(index_notnan_clustprop)==0
    fstarmc_out=nan(T,r,nrep);
    lamstarmc_out=nan(N,r,nrep);
    return
end

% construct clusters
fstarmc_clust=nan(T,r,nrep);
lamstarmc_clust=nan(N,r,nrep);
index_kmedoids_clust=zeros(r,nrep);
for i=1:nrep
    if index_notnan_clustprop(1,i)==1 % only if unique, then fill in factors, else keep nan
        index_kmedoidsr_restacki=index_kmedoidsr_restack(:,i);
        index_kmedoidsr_restacki_notnan=index_kmedoidsr_restacki(not(isnan(index_kmedoidsr_restacki)));
        for j=1:r
            if ismember(j,index_kmedoidsr_restacki_notnan)
                fstarmc_clust(:,j,i)=fstarmc(:,index_kmedoidsr_restacki_notnan==j,i);
                lamstarmc_clust(:,j,i)=lamstarmc(:,index_kmedoidsr_restacki_notnan==j,i);
                index_kmedoids_clust(j,i)=1;
            end
        end
    end
end
% index_kmedoidsr_restack(:,1:5),squeeze(mean(fstarmc(:,:,1:5),1)),squeeze(mean(fstarmc_clust(:,:,1:5),1)),index_kmedoids_clust(:,1:5)

sum(isnan(index_kmedoids_clust),2),not(sum(isnan(index_kmedoids_clust),2)==nrep)
% clean nan factors
index_notnanfactors=not(sum(index_kmedoids_clust==0,2)==nrep);
fstarmc_clust_clean=fstarmc_clust(:,index_notnanfactors,:);
lamstarmc_clust_clean=lamstarmc_clust(:,index_notnanfactors,:);
index_kmedoids_clust_clean=index_kmedoids_clust(index_notnanfactors,:);
r_clust=size(index_kmedoids_clust_clean,1);
disp(['  no of identified and clustered factors ',num2str(r_clust),' (from ',num2str(r),')']);

% remove sign change in each col/cluster
lamstarmc_clust_sign=lamstarmc_clust_clean;
fstarmc_clust_sign=fstarmc_clust_clean;
r2neg_index_mat=nan(r_fix,nrep);
for i=1:r_fix
    lamstarmc_clust_i_pre=squeeze(lamstarmc_clust(:,i,:));
    index_notnan=not(isnan(lamstarmc_clust_i_pre(1,:)));
    lamstarmc_clust_i=lamstarmc_clust_i_pre(:,index_notnan);
    r2=corrcoef([lamstarmc_clust_i(:,end),lamstarmc_clust_i(:,1:end-1)]);
    r2=r2(1,2:end)';
    r2neg_index=logical(r2<0); % [r2mat,r2matneg_index,r2matpos,logical(r2matpos<0)]
    r2neg_index=logical([r2neg_index;zeros(1,1)]); % note: r2 computed relative to last rep!
    r2neg_index_mat(i,1:size(r2neg_index,1))=r2neg_index;
    lamstarmc_clust_i(:,r2neg_index)=lamstarmc_clust_i(:,r2neg_index)*-1.0;
    lamstarmc_clust_i_post=nan(size(lamstarmc_clust_i_pre));
    lamstarmc_clust_i_post(:,index_notnan)=lamstarmc_clust_i;
    lamstarmc_clust_sign(:,i,:)=lamstarmc_clust_i_post;
    % 
    fstarmc_clust_i_pre=squeeze(fstarmc_clust(:,i,:));
    fstarmc_clust_i=fstarmc_clust_i_pre(:,index_notnan);
    fstarmc_clust_i(:,r2neg_index)=fstarmc_clust_i(:,r2neg_index)*-1.0;
    fstarmc_clust_i_post=nan(size(fstarmc_clust_i_pre));
    fstarmc_clust_i_post(:,index_notnan)=fstarmc_clust_i;
    fstarmc_clust_sign(:,i,:)=fstarmc_clust_i_post;
end

lamstarmc_out=lamstarmc_clust_sign(:,:,index_notnan_clustprop);
fstarmc_out=fstarmc_clust_sign(:,:,index_notnan_clustprop);

end
