function [ftmcstar_out,lamstarmc_out,it_sel]= f_permute_factors_group(ftmcstar,lamstarmc,ftstmed)
% factor model identification according to identified factor patterns (run identify_factors_new before)
% 1. correlate the draws with the identified factor pattern, permute accordingly factor-specific parameters and perform sign-identification
%    (i) keep those draws in it_sel.in which are uniquely assigned to the factor patterns based on maximum correlation with factor patterns
%    (ia) determine to which factor pattern combination the draw corresponds to
%    (ii) permute all factor-specific (loadings etc.) parameters according to the assignment of the factor draws to the factor patterns
%    (iii) switch the sign of factor draws and factor-specific parameters which are negatively correlated with the assigned factor patterns 
% 2. perform overall sign switch, to achieve a majority of positive factor loadings 

nf=size(lamstarmc,2);
MMC=size(lamstarmc,3);

it_sel=struct([]);
it_sel(1).out=[];
it_sel(1).in=[];
nm=size(ftstmed,2);
perm_no=nchoosek(nm,nf);
perm_pat=nchoosek(1:nm,nf);
perm_c=zeros(perm_no,MMC)==1;

% 1. correlate the draws with the identified factor pattern, permute accordingly factor-specific parameters and perform sign-identification
for mc=1:MMC;
    co_mat=corr(ftstmed,ftmcstar(:,:,mc));
    co_max=max(abs(co_mat),[],1);
    ind_co=(abs(co_mat)==kron(ones(nm,1),co_max));
    [si,r]=find(ind_co');
    co_sign=sign(co_mat(ind_co));
    co_sign=co_sign(si);
    ind_sig=find(co_sign<0);
    % (i) keep those draws in it_sel.in which are uniquely assigned to the factor patterns based on maximum correlation with factor patterns
    if any([sum(ind_co,1) sum(ind_co,2)']>1)
        it_sel(1).out(end+1)=mc;
    else
        it_sel(1).in(end+1)=mc;
        perm_c(:,mc)=all(r(:,ones(perm_no,1))'==perm_pat,2);
        % (ii) permute all factor-specific (loadings etc.) parameters according to the assignment of the factor draws to the factor patterns
        lamstarmc(:,:,mc)=lamstarmc(:,si,mc);
        ftmcstar(:,:,mc)=ftmcstar(:,si,mc);
        % (iii) switch the sign of factor draws and factor-specific parameters which are negatively correlated with the assigned factor patterns         
        if any(ind_sig)
            lamstarmc(:,ind_sig,mc)=-lamstarmc(:,ind_sig,mc);
            ftmcstar(:,ind_sig,mc)=-ftmcstar(:,ind_sig,mc);
        end
    end
end

it_sel.permc=perm_c;
for c=1:perm_no;
    id=it_sel.in(perm_c(c,it_sel.in)==1);
    % 2. perform overall sign switch, to achieve a majority of positive factor loadings 
    lamstarm=median(lamstarmc(:,:,id),3);
    s_ind=f_checkpositivity(lamstarm);
    for j=s_ind
        lamstarmc(:,j,id)=-lamstarmc(:,j,id);
        ftmcstar(:,j,id)=-ftmcstar(:,j,id);
    end
end

disp('combination of factor pattern and number of occurences');
disp([perm_pat sum(perm_c(:,it_sel.in)==1,2)]);

lamstarmc_out=lamstarmc;
ftmcstar_out=ftmcstar;

end
