function f_forecast_eval(name_out,model_names,options,data)
%
% forecast evaluation 
%

% diary
name = [ 'eval_' name_out '.out' ];
delete([ '.\results\' name ]);
diary([ '.\results\' name ]);
disp(' ');
disp('forecast evaluation');
disp(' ');
disp(['diary file: ', '.\results\' name ]);
disp(['current date: ' datestr(clock,31)]);
disp(['current directory: ' pwd]);
disp(' ');
if isempty(options.sample)
    options.sample = 'all';
    disp(['sample selection method not specified and set to: ', options.sample]);
end
disp(['sample selection method chosen: ', options.sample]);

% use names for loading
n_models = size(model_names,2);
disp(' ');
disp('forecasting models to be compared');
disp(char(model_names));
disp(' ');
disp([ 'number of forecasting models ' num2str(n_models) ]);
disp(' ');

% load forecast errors
for i=1:n_models
    load_name         = [ 'r_' model_names{1,i} ];
    results           = f_loadfind2([load_name,'.mat'],options.loadpaths,'on');
    model(i).forecast = results.forecast;
    model(i).name     = model_names{1,i};
end

fluct_names = model_names;
model_names = char(model_names);

% get data and params from 1st model
data_m           = data.data_m;
data_q           = data.data_q;
data_gdp         = data.data_gdp;
periods_eval     = model(1).forecast.sim_param.periods_eval;
date_start_f_gdp = model(1).forecast.sim_param.date_start_f_gdp;
date_final_f_gdp = model(1).forecast.sim_param.date_final_f_gdp;
date_now_m       = model(1).forecast.sim_param.date_now_m;
h                = model(1).forecast.sim_param.h;

% setup calender
start_year  = date_start_f_gdp(1,1);
start_quart = date_start_f_gdp(1,2);
end_year    = date_final_f_gdp(1,1);
end_quart   = date_final_f_gdp(1,2); 
period = 1;
cal = [start_year,start_quart,period];
i_year  = start_year;
i_quart = start_quart;
while ((i_year ~= end_year) || (i_quart ~= end_quart))
    if (i_quart == 4)
        i_quart = 1;
        i_year = i_year+1;
    else
        i_quart = i_quart+1;
    end
    period = period+1;
    cal = [ cal; [i_year,i_quart,period] ];
end
cal_q = cal;

% disp
disp(' ');
disp('dates and periods: '); 
disp([' pot start date forecast eval gdp:  ', f_date2str_q(date_start_f_gdp)]);
disp([' pot final date forecast gdp:       ', f_date2str_q(date_final_f_gdp)]);
disp([' pot periods for forecast eval:     ', num2str(periods_eval),' quarters']);
disp([' download date:                     ', f_date2str_m(date_now_m)]);

% cut sample
if (strcmp(options.sample,'1sthalf') || strcmp(options.sample,'2ndhalf'))
    t2 = round(periods_eval/2);
elseif (strcmp(options.sample,'1third') || strcmp(options.sample,'23third'))
    t2 = round(periods_eval/3);
elseif (strcmp(options.sample,'1fourth') || strcmp(options.sample,'234fourth'))
    t2 = round(periods_eval/4);
elseif (strcmp(options.sample,'manfirst') || strcmp(options.sample,'manlast')) 
    t2 = options.t2;
elseif (strcmp(options.sample,'manall')) 
    t_index1 = options.t_index(1,1);
    t_index2 = options.t_index(1,2);
elseif (strcmp(options.sample,'manallstr')) 
    date_start = options.t_date(1,1:2);
    date_end   = options.t_date(2,1:2);
    t_index1   = f_find_date(cal_q(:,1:2),date_start);
    t_index2   = f_find_date(cal_q(:,1:2),date_end);
    if isnan(t_index1)
        t_index1 = 1;
    end
    if isnan(t_index2)
        t_index2 = periods_eval;
    end
end
if (strcmp(options.sample,'1sthalf') || strcmp(options.sample,'1third') || strcmp(options.sample,'1fourth') || ...
    strcmp(options.sample,'manfirst'))
    periods_sel = 1:t2;
elseif (strcmp(options.sample,'2ndhalf') || strcmp(options.sample,'23third') || strcmp(options.sample,'234fourth') || ... 
    strcmp(options.sample,'manlast'))
    periods_sel = t2+1:periods_eval;
elseif (strcmp(options.sample,'manall') || strcmp(options.sample,'manallstr'))
    periods_sel = t_index1:t_index2;
elseif (strcmp(options.sample,'all'))
    periods_sel = 1:periods_eval;
end
disp([' effective sample points selected:  ', num2str(periods_sel(1,1)), ' ', num2str(periods_sel(1,end)), ...
      ' (', options.sample, ')']);
disp([' effective sample period selected:  ', num2str(f_date2str_q(cal_q(periods_sel(1,1),1:2))), ...
      ' ', num2str(f_date2str_q(cal_q(periods_sel(1,end),1:2)))]); % disp(cal_q);
  
% loop
if (strcmp(options.print,'on'))
    disp(' '); disp('loop for collecting forecast errors: '); 
end

for k=1:n_models
    if (strcmp(options.print,'on'))
        disp([' model: ', deblank(model_names(k,:))]);
    end
    model(k).forecast.gdp = NaN(h*3,periods_eval);
    model(k).forecast.gdp_for = NaN(h*3,periods_eval);
    model(k).forecast.error = NaN(h*3,periods_eval);
    model(k).forecast.sqerror = NaN(h*3,periods_eval);
    model(k).forecast.abserror = NaN(h*3,periods_eval);
    model(k).forecast.dates_eval = NaN(2,periods_eval);
    date_index_m_mat = [];
    for i=1:periods_eval
        % quarterly date to predict
        date_index_q = f_find_date(data_gdp.dates,date_start_f_gdp)-1+i;
        date_q = data_gdp.dates(date_index_q,:);
        if (strcmp(options.print,'on'))
            disp([' forecast eval period: ', f_date2str_q(date_q), ', date_index: ', num2str(date_index_q), ', i: ', num2str(i)]);
        end
        date_lastmonth = [date_q(1,1),date_q(1,2)*3];
        model(k).forecast.dates_eval(:,i) = date_q';
        % monthly date for information set
        for j=1:(h*3)
            date_index_m = f_find_date(data_m.dates,date_lastmonth)+1-j;
            date_index_m_mat = [ date_index_m_mat; date_index_m ];
            date_m = data_m.dates(date_index_m,:);
            if (strcmp(options.print,'on'))
                disp(['  get forecast with info in month: ', f_date2str_m(date_m), ', date_index: ', num2str(date_index_m), ', j: ', num2str(j)]);
            end
            % find recursion, date, and forecast
            rec_m = f_find_recursion(model(k).forecast.rec,date_m);
            dates_forecast = model(k).forecast.rec(rec_m).dates;
            index_forecast = f_find_date(dates_forecast,date_q);
            gdp_for = model(k).forecast.rec(rec_m).gdp_for(index_forecast,1);
            gdp = model(k).forecast.rec(rec_m).gdp(index_forecast,1);
            % store
            model(k).forecast.gdp(j,i)      = gdp;
            model(k).forecast.gdp_for(j,i)  = gdp_for;
            model(k).forecast.error(j,i)    = gdp-gdp_for;
            model(k).forecast.sqerror(j,i)  = (gdp-gdp_for).^2;
            model(k).forecast.abserror(j,i) = abs(gdp-gdp_for);
        end
    end
    % cut recursions
    model(k).forecast.gdp        = model(k).forecast.gdp(:,periods_sel);
    model(k).forecast.gdp_for    = model(k).forecast.gdp_for(:,periods_sel);
    model(k).forecast.error      = model(k).forecast.error(:,periods_sel);
    model(k).forecast.sqerror    = model(k).forecast.sqerror(:,periods_sel);
    model(k).forecast.abserror   = model(k).forecast.abserror(:,periods_sel);
    model(k).forecast.dates_eval = model(k).forecast.dates_eval(:,periods_sel);
    % average over months
    if (strcmp(options.month_quarter,'quarter'))
        gdp = [];
        gdp_for = [];
        error = [];
        sqerror = [];
        abserror = [];
        for j=1:h
            row1 = (j-1)*h+1;
            row2 = (j-1)*h+h;
            gdp =      [gdp; mean(model(k).forecast.gdp(row1:row2,:),1) ];
            gdp_for =  [gdp_for; mean(model(k).forecast.gdp_for(row1:row2,:),1) ];
            error =    [error; mean(model(k).forecast.error(row1:row2,:),1) ];
            sqerror =  [sqerror; mean(model(k).forecast.sqerror(row1:row2,:),1) ];
            abserror = [abserror; mean(model(k).forecast.abserror(row1:row2,:),1) ];
        end
        model(k).forecast.gdp      = gdp;
        model(k).forecast.gdp_for  = gdp_for;
        model(k).forecast.error    = error;
        model(k).forecast.sqerror  = sqerror;
        model(k).forecast.abserror = abserror;
    end
    % average over recursions
    model(k).forecast.gdp_mean = mean(model(k).forecast.gdp,2);
    model(k).forecast.gdp_var  = mean((model(k).forecast.gdp-kron(ones(1,size(model(k).forecast.gdp,2)),model(k).forecast.gdp_mean)).^2,2);
    model(k).forecast.gdp_sd   = sqrt(mean((model(k).forecast.gdp-kron(ones(1,size(model(k).forecast.gdp,2)),model(k).forecast.gdp_mean)).^2,2));
    model(k).forecast.gdp_var0 = mean((model(k).forecast.gdp).^2,2);
    model(k).forecast.gdp_sd0  = sqrt(mean((model(k).forecast.gdp).^2,2));
    model(k).forecast.mse      = mean(model(k).forecast.sqerror,2);
    model(k).forecast.rmse     = sqrt(mean(model(k).forecast.sqerror,2));
    model(k).forecast.mae      = mean(model(k).forecast.abserror,2);
end
h_tab = size(model(1).forecast.gdp,1);

% comp mse,mae matrices
mse  = [];
rmse = [];
mae  = [];
for k=1:n_models
    mse  = [ mse; model(k).forecast.mse' ];
    rmse = [ rmse; model(k).forecast.rmse' ];
    mae  = [ mae; model(k).forecast.mae' ];
end

% save forecast results
results_all.mse   = mse;
results_all.rmse  = rmse;
results_all.mae   = mae;
results_all.names = model_names;

% store forecasts for mcs
for j = 1:h_tab
    names = {'gdp'};
    gdp_forecast = model(1).forecast.gdp(j,:);
    for k = 1:n_models
        gdp_forecast = [gdp_forecast; model(k).forecast.gdp_for(j,:)];
        names = [names, model(k).name];
    end
    mResults.gdp_gdp_for = gdp_forecast';
    mResults.names = names;
    sOut(j).mResults = mResults;
end

% disp mse
digs  = 3;
scale = 1.0;
disp(' '); disp('mse (row order: horizons, ... )'); disp(' ');
disp([ [ blanks(size(model_names,2)); model_names ], f_mat2str(mse*scale,[1:h_tab],digs) ]); disp(' '); 
disp('variance predictand and zero forecast variance'); 
var_gdp = [ model(1).forecast.gdp_var'; model(1).forecast.gdp_var0' ];
disp( [ [ blanks(size(model_names,2)); blanks(size(model_names,2)); blanks(size(model_names,2)) ], f_mat2str(var_gdp*scale,[1:h_tab],digs) ] ); disp(' '); 

% disp relative mse to variance
disp(' '); disp('mse relative to gdp variance (row order: horizons, ... )'); disp(' ');
mat = mse./kron(ones(n_models,1),model(1).forecast.gdp_var');
disp([ [ blanks(size(model_names,2)); model_names ], f_mat2str(mat,[1:h_tab],2) ]); disp(' '); 
% export latex
dum_latex = 'off'; 
if (strcmp(dum_latex,'on'))
    cut = 4;
    tab_show = cell(1,1);
    names_mat = model_names(:,cut+1:end);
    tab_show(1,1) = cellstr(' ');    
    tab_show(4,1) = cellstr('horizon month');    
    for h=1:h_tab
        if (h == 1)
            tab_show(2,1+h) = cellstr('nowcast');
            tab_show(3,1+h) = cellstr('current quarter');
        elseif (h >= 4 && round(1+(h-1)/3)==1+(h-1)/3)
            tab_show(2,1+h) = cellstr(['forecast']);
            tab_show(3,1+h) = cellstr([num2str(1+(h-1)/3)-1,' quarter(s)']);
        end
        tab_show(4,1+h) = cellstr(num2str(h,['%10.0f']));
        for j=1:n_models
            tab_show(5+j,1)   = cellstr(names_mat(j,:));
            tab_show(5+j,1+h) = cellstr(num2str(mat(j,h),['%10.2f']));    
        end
    end
    tab_show(5+n_models+1,1) = cellstr(' ');    
    tablename = 'mse relative to gdp variance';
    name_in = 'mserelvar';
    savename = ['tab_',name_out,'_',name_in,'.tex'];
    f_cell2swp(tab_show,savename,tablename);
end

% disp rmse
digs  = 3;
scale = 1.0;
disp(' '); disp('rmse (row order: horizons, ... )'); disp(' ');
disp([ [ blanks(size(model_names,2)); model_names ], f_mat2str(rmse*scale,[1:h_tab],digs) ]); disp(' '); 
disp('sd predictand and zero forecast variance'); 
sd_gdp = [ model(1).forecast.gdp_sd'; model(1).forecast.gdp_sd0' ];
disp( [ [ blanks(size(model_names,2)); blanks(size(model_names,2)); blanks(size(model_names,2)) ], f_mat2str(sd_gdp*scale,[1:h_tab],digs) ] ); disp(' '); 

% disp relative rmse to sd
disp(' '); disp('rmse relative to gdp sd (row order: horizons, ... )'); disp(' ');
mat = rmse./kron(ones(n_models,1),model(1).forecast.gdp_sd');
disp([ [ blanks(size(model_names,2)); model_names ], f_mat2str(mat,[1:h_tab],2) ]); disp(' '); 
% export latex
dum_latex = 'off'; 
if (strcmp(dum_latex,'on'))
    cut = 4;
    tab_show = cell(1,1);
    names_mat = model_names(:,cut+1:end);
    tab_show(1,1) = cellstr(' ');    
    tab_show(4,1) = cellstr('horizon month');    
    for h=1:h_tab
        if (h == 1)
            tab_show(2,1+h) = cellstr('nowcast');
            tab_show(3,1+h) = cellstr('current quarter');
        elseif (h >= 4 && round(1+(h-1)/3)==1+(h-1)/3)
            tab_show(2,1+h) = cellstr(['forecast']);
            tab_show(3,1+h) = cellstr([num2str(1+(h-1)/3)-1,' quarter(s)']);
        end
        tab_show(4,1+h) = cellstr(num2str(h,['%10.0f']));
        for j=1:n_models
            tab_show(5+j,1)   = cellstr(names_mat(j,:));
            tab_show(5+j,1+h) = cellstr(num2str(mat(j,h),['%10.2f']));    
        end
    end
    tab_show(5+n_models+1,1) = cellstr(' ');    
    tablename = 'rmse relative to gdp sd';
    name_in = 'rmserelvar';
    savename = ['tab_',name_out,'_',name_in,'.tex'];
    f_cell2swp(tab_show,savename,tablename);
end

% rmse quantile
rmse_quantile = NaN(n_models,h_tab); 
for i=1:h_tab
    for j=1:n_models
        rmse_quantile(j,i) = f_invquant(rmse(:,i),0.0:0.01:1.0,rmse(j,i));
    end    
end
disp(' '); disp('rmse quantile (row order: horizons, ... )'); disp(' ');
mat = rmse_quantile;
disp([ [ blanks(size(model_names,2)); model_names ], f_mat2str(mat,[1:h_tab],2) ]); disp(' '); 

% mse ranking
[mse_sort,mse_index] = sort(mse); 
mse_ranking = NaN(n_models,h_tab); 
for i=1:h_tab
    for j=1:n_models
        mse_ranking(mse_index(j,i),i) = j;
    end    
end
[mse_sort,mse_index_min] = min(mse); 
disp(' '); disp(['mse ranking (lowest mse = 1, highest mse = ' num2str(n_models) ')']); disp(' ');
mat = mse_ranking;
disp( [ [ blanks(size(model_names,2)); model_names ], f_mat2str(mat,[1:h_tab],0) ]); disp(' '); 
disp('index of best method'); 
disp( [ [ blanks(size(model_names,2)); blanks(size(model_names,2)) ], f_mat2str(mse_index_min,[1:h_tab],0) ] ); disp(' '); 
% export latex
dum_latex = 'off'; 
if (strcmp(dum_latex,'on'))
    cut = 4;
    tab_show = cell(1,1);
    names_mat = model_names(:,cut+1:end);
    tab_show(1,1) = cellstr(' ');    
    tab_show(4,1) = cellstr('horizon month');    
    for h=1:h_tab
        if (h == 1)
            tab_show(2,1+h) = cellstr('nowcast');
            tab_show(3,1+h) = cellstr('current quarter');
        elseif (h >= 4 && round(1+(h-1)/3)==1+(h-1)/3)
            tab_show(2,1+h) = cellstr(['forecast']);
            tab_show(3,1+h) = cellstr([num2str(1+(h-1)/3)-1,' quarter(s)']);
        end
        tab_show(4,1+h) = cellstr(num2str(h,['%10.0f']));
        for j=1:n_models
            tab_show(5+j,1)   = cellstr(names_mat(j,:));
            tab_show(5+j,1+h) = cellstr(num2str(mat(j,h),['%10.0f']));    
        end
    end
    tab_show(5+n_models+1,1) = cellstr(' ');    
    tablename = 'mse ranking';
    name_in   = 'rank';
    savename  = ['tab_',name_out,'_',name_in,'.tex'];
    f_cell2swp(tab_show,savename,tablename);
end

% disp relative rmse to rmse of selected model
if (not(isempty(options.benchmark_model_index)))
    index = options.benchmark_model_index;
    disp(' '); disp(['rmse relative to rmse of benchmark model ', deblank(model_names(index,:)), ' (row order: horizons, ... )']); disp(' ');
    mat = rmse./kron(ones(n_models,1),rmse(index,:));
    disp([ [ blanks(size(model_names,2)); model_names ], f_mat2str(mat,[1:h_tab],2) ]); disp(' '); 
    % export latex
    dum_latex = 'off'; 
    if (strcmp(dum_latex,'on'))
        cut = 4;
        tab_show = cell(1,1);
        names_mat = model_names(:,cut+1:end);
        tab_show(1,1) = cellstr(' ');    
        tab_show(4,1) = cellstr('horizon month');    
        for h=1:h_tab
            if (h == 1)
                tab_show(2,1+h) = cellstr('nowcast');
                tab_show(3,1+h) = cellstr('current quarter');
            elseif (h >= 4 && round(1+(h-1)/3)==1+(h-1)/3)
                tab_show(2,1+h) = cellstr(['forecast']);
                tab_show(3,1+h) = cellstr([num2str(1+(h-1)/3)-1,' quarter(s)']);
            end
            tab_show(4,1+h) = cellstr(num2str(h,['%10.0f']));
            for j=1:n_models
                tab_show(5+j,1)   = cellstr(names_mat(j,:));
                tab_show(5+j,1+h) = cellstr(num2str(mat(j,h),['%10.2f']));    
            end
        end
        tab_show(5+n_models+1,1) = cellstr(' ');    
        tablename = ['rmse relative to reference model',deblank(model_names(index,:))];
        name_in   = 'rmserelmod';
        savename  = ['tab_',name_out,'_',name_in,'.tex'];
        f_cell2swp(tab_show,savename,tablename);
    end
end

% disp relative mse to mse of selected model
if (not(isempty(options.benchmark_model_index)))
    index = options.benchmark_model_index;
    disp(' '); disp(['mse relative to mse of benchmark model ', deblank(model_names(index,:)), ' (row order: horizons, ... )']); disp(' ');
    mat = mse./kron(ones(n_models,1),mse(index,:));
    disp([ [ blanks(size(model_names,2)); model_names ], f_mat2str(mat,[1:h_tab],2) ]); disp(' '); 
    % export latex
    dum_latex = 'off'; 
    if (strcmp(dum_latex,'on'))
        cut = 4;
        tab_show = cell(1,1);
        names_mat = model_names(:,cut+1:end);
        tab_show(1,1) = cellstr(' ');    
        tab_show(4,1) = cellstr('horizon month');    
        for h=1:h_tab
            if (h == 1)
                tab_show(2,1+h) = cellstr('nowcast');
                tab_show(3,1+h) = cellstr('current quarter');
            elseif (h >= 4 && round(1+(h-1)/3)==1+(h-1)/3)
                tab_show(2,1+h) = cellstr(['forecast']);
                tab_show(3,1+h) = cellstr([num2str(1+(h-1)/3)-1,' quarter(s)']);
            end
            tab_show(4,1+h) = cellstr(num2str(h,['%10.0f']));
            for j=1:n_models
                tab_show(5+j,1)   = cellstr(names_mat(j,:));
                tab_show(5+j,1+h) = cellstr(num2str(mat(j,h),['%10.2f']));    
            end
        end
        tab_show(5+n_models+1,1) = cellstr(' ');    
        tablename = ['mse relative to reference model',deblank(model_names(index,:))];
        name_in   = ['rmserelmod'];
        savename  = ['tab_',name_out,'_',name_in,'.tex'];
        f_cell2swp(tab_show,savename,tablename);
    end
end

% compare elements of groups by identifiers
if (isfield(options,'compare_groups'))
if (strcmp(options.compare_groups.switch,'on'))
    %     options.compare_groups.switch    = 'off';
    %     options.compare_groups.switch2   = 'off';
    %     options.compare_groups.group_rel = {'mid'};
    %     options.compare_groups.groups    = {'midu'};
    groups = options.compare_groups.groups;
    if (size(groups,2) ~= 2)
        disp('compare elements of groups: groups not correctly identified!'); disp(' ');
    else
        name1 = groups{1,1};
        name2 = groups{1,2};
        name1_index = [];
        name2_index = [];
        for j=1:n_models
            name_model  = model_names(j,:);
            name1_index = [name1_index, not(isempty(findstr(name_model, name1))) ];
            name2_index = [name2_index, not(isempty(findstr(name_model, name2))) ];
        end
        %disp([(1:n_models)',name1_index',name2_index']);
        if (all(name1_index == 0) || all(name2_index == 0))
            disp('compare elements of groups: groups not correctly identified!'); disp(' ');
        else
            name1_index = find(name1_index);
            name2_index = find(name2_index);
            %disp([name1_index',name1_index']);
            model_pairs = [];
            model_names_sel = [];
            for j=1:size(name1_index,2)
                index        = name1_index(1,j);
                name1_model  = model_names(index,:);
                index1 = findstr(name1, name1_model);
                index2 = index1+size(name1,2)-1;
                name1_model_comp = deblank(name1_model(1,setdiff([1:size(name1_model,2)],index1:index2)));
                for k=1:size(name2_index,2)
                    index        = name2_index(1,k);
                    name2_model  = model_names(index,:);
                    index1 = findstr(name2, name2_model);
                    index2 = index1+size(name2,2)-1;
                    name2_model_comp = deblank(name2_model(1,setdiff([1:size(name2_model,2)],index1:index2)));
                    if (strcmp(name1_model_comp,name2_model_comp))
                        model_pairs = [ model_pairs; name1_index(1,j),name2_index(1,k) ];
                        name2_model_comp = deblank(name2_model_comp);
                        model_names_sel = char(model_names_sel,name2_model_comp(1,1:end));
                    end
                end
            end
            model_names_sel = model_names_sel(2:end,:);
            disp(' '); disp(['mse for group ''',name1,''' relative to mse of group ''',name2,'''']); disp(' ');
            mat = mse(model_pairs(:,1),:)./mse(model_pairs(:,2),:);
            disp([ [ blanks(size(model_names_sel,2)); model_names_sel ], f_mat2str(mat,[1:h_tab],2) ]); disp(' '); 
        end
    end
    % export latex
    dum_latex = 'off'; 
    if (strcmp(dum_latex,'on'))
        cut = 4;
        tab_show = cell(1,1);
        names_mat = model_names(:,cut+1:end);
        tab_show(1,1) = cellstr(' ');    
        tab_show(4,1) = cellstr('horizon month');    
        for h=1:h_tab
            if (h == 1)
                tab_show(2,1+h) = cellstr('nowcast');
                tab_show(3,1+h) = cellstr('current quarter');
            elseif (h >= 4 && round(1+(h-1)/3)==1+(h-1)/3)
                tab_show(2,1+h) = cellstr(['forecast']);
                tab_show(3,1+h) = cellstr([num2str(1+(h-1)/3)-1,' quarter(s)']);
            end
            tab_show(4,1+h) = cellstr(num2str(h,['%10.0f']));
            for j=1:size(mat,1)
                tab_show(5+j,1)   = cellstr(names_mat(j,:));
                tab_show(5+j,1+h) = cellstr(num2str(mat(j,h),['%10.2f']));    
            end
        end
        tab_show(5+size(mat,1)+1,1) = cellstr(' ');    
        tablename = ['mse for group ''',name1,''' relative to mse of group ''',name2,''''];
        name_in   = ['mserelgroup_',name1,'_',name2];
        savename  = ['tab_',name_out,'_',name_in,'.tex'];
        f_cell2swp(tab_show,savename,tablename);
    end
end
end

% count better method over observations
if (isfield(options,'cou'))
if (strcmp(options.cou,'on'))
    cou = NaN(n_models,n_models,h); 
    disp(' '); 
    disp('percent of observations where one method has smaller error than other method'); disp(' ');
    disp(model_names); disp(' '); 
    for i = 1:h_tab
        for j = 1:n_models
            for k = 1:n_models
                eindi1 = model(j).forecast.sqerror(i,:)';
                eindi2 = model(k).forecast.sqerror(i,:)';
                if k ~= j
                    index = find(eindi1<eindi2);
                    cou(j,k,i) = size(index,1)/size(eindi2,1); 
                end
            end    
        end    
        disp(['horizon ' num2str(i)]); disp(cou(:,:,i)); 
    end
    disp('note: mod in row has smaller error than mod in column!');
    % export latex
    dum_latex = 'off'; 
    if (strcmp(dum_latex,'on'))
        cut = 4;
        tab_show = cell(1,1);
        names_mat = model_names(:,cut+1:end);
        tab_show(1,1) = cellstr(' ');    
        start_row = 2;
        for h=1:h_tab
            tab_show(start_row,1) = cellstr(['horizon ',num2str(h,['%10.0f'])]);
            for j=1:n_models
                tab_show(start_row,1+j) = cellstr(names_mat(j,:));
            end
            for j=1:n_models
                for k=1:n_models
                    tab_show(start_row+j,1)   = cellstr(names_mat(j,:));
                    tab_show(start_row+j,1+k) = cellstr(num2str(cou(j,k,h),['%10.2f']));    
                end
            end
            start_row = start_row+n_models+2;
        end
        index_fin = size(tab_show,1);
        tab_show(index_fin+1,1) = cellstr(' ');    
        tablename = 'count statistic';
        name_in   = 'count';
        savename  = ['tab_',name_out,'_',name_in,'.tex'];
        f_cell2swp(tab_show,savename,tablename);
    end
end
end

% dm and hln test for equal forecast accuracy
if (isfield(options,'test'))
if (strcmp(options.test,'on'))
    dm =  NaN(n_models,n_models,h); dmsign  = NaN(n_models,n_models,h); 
    hln = NaN(n_models,n_models,h); hlnsign = NaN(n_models,n_models,h); 
    for i = 1:h_tab
        for j = 1:n_models
            for k = 1:n_models
                eindi1 = model(j).forecast.error(i,:)';
                eindi2 = model(k).forecast.error(i,:)';
                if k ~= j
                    modified = 1;
                    [dm(j,k,i),dmsign(j,k,i)] = f_dmtest(eindi1,eindi2,i,modified);
                    [hln(j,k,i),hlnsign(j,k,i)] = f_hlntest(eindi1,eindi2,i,modified);
                end
            end    
        end    
    end
    if (isfield(options,'benchmark_model_index'))
        index = options.benchmark_model_index;
        disp(' '); disp(['uncond giacomini white test for forecasting accuracy (sign level) between models and benchmark ', ...
                         deblank(model_names(index,:)), ' (row order: horizons, ... )']); disp(' ');
        model2_index = setdiff(1:n_models,index);
        mat = NaN(n_models-1,h_tab);
        for i = 1:h_tab
            mat(:,i) = dmsign(model2_index,index,i);
        end
        disp([ [ blanks(size(model_names(model2_index,:),2)); model_names(model2_index,:) ], f_mat2str(mat,[1:h_tab],2) ]); disp(' '); 
    else
        disp(' '); disp('uncond giacomini white (diebold mariano) test for forecasting accuracy (sign level)'); disp(' ');
        disp(model_names); disp(' '); 
        for i = 1:h_tab
            disp(['horizon ' num2str(i)]); disp(dmsign(:,:,i)); 
        end
        disp(' '); disp('hln test for forecast encompassing (sign level)'); disp(' ');
        disp(model_names); disp(' '); 
        for i = 1:h_tab
            disp(['horizon ' num2str(i)]); disp(hlnsign(:,:,i)); 
        end
        disp('h0: mod in row encompasses mod in column (lambda = 0)!');
    end
end
end
results_all.dm.pval = [];
results_all.dm.names = [];
if (isfield(options,'test'))
if (strcmp(options.test,'on'))
    if (isfield(options,'benchmark_model_index'))
        results_all.dm.pval = mat;
        results_all.dm.names = model_names(model2_index,:);
    end
end
end

% store table results
name_out2 = 'all';
savename = [ '.\results\r_' name_out, '_', name_out2 ];
save(savename,'results_all');

if (isfield(options,'fluctuation'))
if (strcmp(options.fluctuation.onoff,'on'))
    
    % options.fluctuation.onoff = 'off';
    % options.fluctuation.window = 5;
    % options.fluctuation.horizon = 1;
    
    h_fluc           = options.fluctuation.horizon;                    % forecasting horizon
    k_benchmark      = options.benchmark_model_index;                  % index of benchmark model 
    errors_benchmark = model(k_benchmark).forecast.error(h_fluc,:)';   % forecast errors of the benchmark  
    m_window         = options.fluctuation.window;                     % the size of rolling window 
    
    even_test = m_window/2;                           % testing if the window size is even
    even_test = round(even_test);                     % testing if the window size is even
    even_test = even_test*2;                          % testing if the window size is even
    
    even_test = m_window - even_test;                 % testing if the window size is even
    
    if(even_test == 0)
        m_window = m_window + 1;                      % only odd-numbered window size is allowed
    end
    
    fluc_mu          = m_window/size(errors_benchmark,1);               
    fluc_mu          = fluc_mu*10;
    fluc_mu          = round(fluc_mu);
    fluc_mu          = fluc_mu/10;                                    % calculation the relative size of rolling window 
    
    if(fluc_mu == 0)
        fluc_mu = 0.1;                                                % infimum of the relative window size
    end
    
    if(fluc_mu == 1)
        fluc_mu = 0.9;                                                % supremum of the relative window size
    end
    
    fluc_cv = [0.1,2.928;                                             % critical values, see Giacomini and Rossi (2008), page 31. 
               0.2,2.676;
               0.3,2.482;
               0.4,2.334;
               0.5,2.168;
               0.6,2.030;
               0.7,1.904;
               0.8,1.740;
               0.9,1.600];
           
    for v = 1:size(fluc_cv,1)                                        % searching for an appropriate critical value 
        if(fluc_mu == fluc_cv(v,1))
            fluc_val = fluc_cv(v,2);
        end
    end
    
    dm_fluct_mean      = NaN(size(errors_benchmark,1)-m_window+1,1);              % defining a series for the dm-statistic 
    fluc_val_series    = ones(size(errors_benchmark,1)-m_window+1,1)*fluc_val;    % defining a series with the critical value
    fluc_val_series    = -1*fluc_val_series;
    
    % ***producing ticks for plotting***
    dates_eval = model(1).forecast.dates_eval;
    dates_graph = [];
    for i=1:size(dates_eval,2)
        year_str = num2str(dates_eval(1,i));
        year_str = year_str(1,3:4);
        quarter_str = num2str(dates_eval(2,i));
        dates_graph = char(dates_graph, [  year_str 'q' quarter_str ]);
    end
    
    cut_number = (m_window - 1)/2; 
    
    dates_graph = dates_graph((2+cut_number):(end-cut_number),:);
     
    % which ticks to name?
    dates_ticks = [2:2:size(dates_eval(:,cut_number:(end-cut_number)),2)-1];
    % *********************************
    
    for v = 1:n_models
        if(v ~= k_benchmark)
            errors_model = model(v).forecast.error(h_fluc,:)';                    % picking up model errors
            %diff_errors = (errors_benchmark - errors_model).^2;
            
            for w = 1:size(errors_model,1) - m_window + 1                         % moving window
                smpl_first = w;
                smpl_last  = w + m_window - 1;
                dm_benchmark     = errors_benchmark(smpl_first:smpl_last);        % picking up the errors for the dm-test
                dm_model         = errors_model(smpl_first:smpl_last);            % picking up the errors for the dm-test
                [dm_fluct_mean(w) dm_sign] = f_dmtest(dm_model,dm_benchmark,h_fluc,0);  % dm-test
            end
            
            colordef white;
            set(0,'DefaultAxesLineStyleOrder',{'-','--','-.',':','-v','--^'});
            set(0,'DefaultAxesColorOrder',[0.0,0.0,0.0])
            figure; newplot;
            plot([dm_fluct_mean fluc_val_series]);                                % plotting 
            set(gca,'XTick',dates_ticks);
            set(gca,'XTickLabel',dates_graph(dates_ticks,:));
            set(gca,'XTickMode','manual');
            set(gca,'FontSize',8);
            title([char(fluct_names(k_benchmark)), ' vs. ', char(fluct_names(v))],'Interpreter','none');  % plotting 
        end
    end
    
end    
end

% figures
if (isfield(options,'figures'))
if (strcmp(options.figures.onoff,'on'))
    % prepare ticks for plotting
    dates_eval = model(1).forecast.dates_eval;
    dates_graph = [];
    for i=1:size(dates_eval,2)
        year_str = num2str(dates_eval(1,i));
        year_str = year_str(1,3:4);
        quarter_str = num2str(dates_eval(2,i));
        dates_graph = char(dates_graph, [  year_str 'q' quarter_str ]);
    end
    dates_graph = dates_graph(2:end,:);
    % which ticks to name?
    dates_ticks = [2:2:size(dates_eval,2)-1];
    % plot figures: real-time nowcasts, and squared errors
    for i=1:h_tab
        colordef white;
        set(0,'DefaultAxesLineStyleOrder',{'-','--','-.',':','-v','--^'});
        set(0,'DefaultAxesColorOrder',[0.0,0.0,0.0])
        figure; newplot;
        subplot(2,1,1);
        nowcast = [];
        for j=1:n_models
            nowcast = [ nowcast; model(j).forecast.gdp_for(i,:) ];
        end
        plot(nowcast');
        set(gca,'XTick',dates_ticks);
        set(gca,'XTickLabel',dates_graph(dates_ticks,:));
        set(gca,'XTickMode','manual');
        set(gca,'FontSize',8);
        hold on
        final = model(j).forecast.gdp(i,:);
        plot(final','x');
        hold off
        if (i == i)
            str = model_names(:,5:size(model_names,2));
            str = char(str,'final gdp');
            hplot = legend(str,'Location','Best','Orientation','vertical'); % h = legend(namestrmat2);
            set(hplot,'Interpreter','none');
            legend('boxoff');
            set(gca,'FontSize',8);
        end
        title(['horizon ', num2str(i), ': forecasts and gdp']);
        subplot(2,1,2);
        mse = [];
        for j=1:n_models
            mse = [ mse; model(j).forecast.sqerror(i,:) ];
        end
        plot(mse');
        set(gca,'XTick',dates_ticks);
        set(gca,'XTickLabel',dates_graph(dates_ticks,:));
        set(gca,'XTickMode','manual');
        set(gca,'FontSize',8);
        if (i == i)
            hplot = legend(model_names(:,5:size(model_names,2)),'Location','Best','Orientation','vertical'); % h = legend(namestrmat2);
            set(hplot,'Interpreter','none');
            legend('boxoff');
            set(gca,'FontSize',8);
        end
        title(['horizon ', num2str(i), ': se']);
        set(gcf,'name','forecasts realisations of gdp and squared errors');
        set(gcf,'PaperType','a4');
        set(gcf,'PaperOrientation','portrait');
        set(gcf,'PaperUnits','centimeters');
        set(gcf,'PaperPositionMode','auto');
        set(0,'DefaultAxesLineStyleOrder','remove');
        set(0,'DefaultAxesColorOrder','remove');
    end    
end
end

%
% mcs
if (isfield(options,'mcs'))
if (strcmp(options.mcs.onoff,'on'))
    mcs_all.names = model_names;
    diary off;
    disp(' ');
    for index = 1:h_tab
        %
        disp(' ');
        disp(['h = ',num2str(index),' from ',num2str(h_tab)]);
        %
        mResults = sOut(index).mResults;
        %
        dir_test = exist('ox_transfer','dir');
        if(dir_test == 7)
            rmdir('ox_transfer','s');
        end
        mkdir('ox_transfer');
        cd ox_transfer;
        %
        max_file_size = 100;
        smpl_size = size(mResults.gdp_gdp_for,1);
        xlswrite('smpl_size',smpl_size,'','A1');
        %
        if(size(mResults.names,2) <= max_file_size)
            xlswrite('saved_forecasts_1',mResults.names,'','A1');
            xlswrite('saved_forecasts_1',mResults.gdp_gdp_for,'','A2');
            save_matrix = 1;
            xlswrite('n_files',save_matrix,'','A1');
        else
            n_files = floor(size(mResults.names,2)/max_file_size);
            for i = 1:n_files
                index_right = max_file_size*i;
                index_left  = max_file_size*(i-1)+1;
                save_array  = mResults.names(index_left:index_right);
                save_matrix = mResults.gdp_gdp_for(:,index_left:index_right);
                xlswrite(['saved_forecasts_',num2str(i)],save_array,'','A1');
                xlswrite(['saved_forecasts_',num2str(i)],save_matrix,'','A2');
            end
            save_array = mResults.names(index_right+1:end);
            save_matrix = mResults.gdp_gdp_for(:,index_right+1:end);
            xlswrite(['saved_forecasts_',num2str(n_files+1)],save_array,'','A1');
            xlswrite(['saved_forecasts_',num2str(n_files+1)],save_matrix,'','A2');
            save_matrix = n_files + 1;
            xlswrite('n_files',save_matrix,'','A1');
        end
        %
        MSCparam = NaN(3,1);
        MSCparam(1) = 0.1;    % msc confidence value
        MSCparam(2) = 6;      % bootstrap block length
        MSCparam(3) = 2000;    % number of bootstrap replications
        xlswrite('msc_param',MSCparam,'','A1');
        cd ..;
        s = dos('C:\Programme\OxMetrics5\Ox\bin\oxl loading.ox');
        s = dos('C:\Programme\OxMetrics5\Ox\bin\oxl mulcom_eval.ox');
        resMCS = xlsread('MCSinfo-Allresults','MCSinfo-Allresults');
        mcs_all.mcsinfo(index).recMCS = resMCS; 
    end
    diary on;
    % sorting the MCS results 
    mcs_final.p_val = []; mcs_final.mse = [];
    for index = 1:h_tab
        mcs_final.p_val = [mcs_final.p_val mcs_all.mcsinfo(index).recMCS(:,6)];
        mcs_final.mse   = [mcs_final.mse mcs_all.mcsinfo(index).recMCS(:,2)];
    end
    % pick up the MCSs for each forecast horizon
    for index = 1:h_tab
        mcs_array_090 = {};
        mcs_array_075 = {};
        for k = 1:size(mcs_final.p_val,1)
            if(mcs_final.p_val(k,index) > 0.1)
                mcs_array_090 = [mcs_array_090 model_names(k)];
            end
            if(mcs_final.p_val(k,index) > 0.25)
                mcs_array_075 = [mcs_array_075 model_names(k)];
            end     
        end
        mcs_final.mcs(index).mcs090 = mcs_array_090;
        mcs_final.mcs(index).mcs075 = mcs_array_075;
    end
    % determining the number of members in each MCS
    mcs_final.mcs_size = [];
    for index = 1:h_tab
        mcs_final.mcs_size = [mcs_final.mcs_size [size(mcs_final.mcs(index).mcs090,2); size(mcs_final.mcs(index).mcs075,2)]];
    end
    % output
    disp(' '); disp(['mcs p-values based on params: confidence value ',num2str(MSCparam(1)), ...
        ', bootstrap block length ',num2str(MSCparam(2)),', bootstraps ',num2str(MSCparam(3))]); disp(' ');
    mat = mcs_final.p_val(options.mcs.modsel_vec,:);
    disp([ [ blanks(size(model_names,2)); model_names(options.mcs.modsel_vec,:) ], f_mat2str(mat,[1:h_tab],2) ]); disp(' '); 
    disp(' '); disp(['mcs number of models included, based on params: confidence value ',num2str(MSCparam(1)), ...
        ', bootstrap block length ',num2str(MSCparam(2)),', bootstraps ',num2str(MSCparam(3))]); disp(' ');
    nmcs_mat = [mcs_final.mcs_size; ones(1,h_tab).*size(mcs_all.names,1)];
    nmcs_names = char({'nmcs90','nmcs75','nallmodels'});
    disp([ [ blanks(size(nmcs_names,2)); nmcs_names ], f_mat2str(nmcs_mat,[1:h_tab],0) ]); disp(' '); 
    % export latex
    dum_latex = 'on';
    if (strcmp(dum_latex,'on'))
        n_models = size(mat,1);
        cut = 4;
        tab_show = cell(1,1);
        names_mat = model_names(options.mcs.modsel_vec,cut+1:end);
        tab_show(1,1) = cellstr(' ');    
        tab_show(4,1) = cellstr('horizon month');    
        for h=1:h_tab
            if (h == 1)
                tab_show(2,1+h) = cellstr('nowcast');
                tab_show(3,1+h) = cellstr('current quarter');
            elseif (h >= 4 && round(1+(h-1)/3)==1+(h-1)/3)
                tab_show(2,1+h) = cellstr(['forecast']);
                tab_show(3,1+h) = cellstr([num2str(1+(h-1)/3)-1,' quarter(s)']);
            end
            tab_show(4,1+h) = cellstr(num2str(h,['%10.0f']));
            for j=1:n_models
                tab_show(5+j,1)   = cellstr(names_mat(j,:));
                tab_show(5+j,1+h) = cellstr(num2str(mat(j,h),['%10.2f']));    
            end
            tab_show(5+n_models+1,1) = cellstr(' ');    
            for j=1:3
                tab_show(5+n_models+1+j,1) = cellstr(nmcs_names(j,:));
                tab_show(5+n_models+1+j,1+h) = cellstr(num2str(nmcs_mat(j,h),['%10.0f']));    
            end
        end
        tab_show(5+n_models+3+1+1,1) = cellstr(' ');    
        %
        tablename = ['mcs p-values'];
        name_in   = ['mcspval'];
        savename  = ['tab_',name_out,'_',name_in,'.tex'];
        f_cell2swp(tab_show,savename,tablename);
    end
end
end

% diary
disp(' '); 
disp('forecast evaluation done.'); 
disp(' '); 
diary off;

return

%
% functions
%

function rec_m = f_find_recursion(recs,date_m)
for i=1:size(recs,2)
    if ( date_m == recs(i).date_m_current )
        rec_m = i;
        return
    end
end
disp('recursion not found!');
return

function b = f_mat2str(a,header,digs)
c = size(a,2);
b = [];
for i=1:c
    a_col = char(num2str(header(1,i), '%10.0f'),num2str(a(:,i), ['%10.',num2str(digs),'f']));
    a_col = strjust(a_col,'right');
    b     = [b,blanks(size(a_col,1))',blanks(size(a_col,1))',a_col];
end
return

function f_cell2swp(tab_in,filename,tablename)
[nrows,ncols] = size(tab_in);
cc = repmat('c',1,ncols);
fid = fopen(filename,'w');
fprintf(fid,'\n\\documentclass{article} \n');
fprintf(fid,'%%TCIDATA{OutputFilter=LATEX.DLL} \n');
fprintf(fid,'%%TCIDATA{Version=4.00.0.2321} \n');
fprintf(fid,'%%TCIDATA{Created=Tuesday, March 11, 2003 22:13:46} \n');
fprintf(fid,'%%TCIDATA{LastRevised=Tuesday, March 11, 2003 22:32:46} \n');
fprintf(fid,'%%TCIDATA{<META NAME="GraphicsSave" CONTENT="32">} \n');
fprintf(fid,'%%TCIDATA{<META NAME="DocumentShell" CONTENT="Standard LaTeX\\Blank - Standard LaTeX Article">} \n');
fprintf(fid,'%%TCIDATA{CSTFile=40 LaTeX article.cst} \n');
fprintf(fid,'\\input{tcilatex} \n');
fprintf(fid,'\\begin{document} \n');
fprintf(fid,['\\caption{',tablename,'\\label{key}} \n']);
fprintf(fid,'\\begin{tabular}{%s} \n',cc);
text_mat = [];
for i = 1:nrows
    text_row = [];
    for j = 1:ncols
        oneelement = tab_in{i,j};
        if (ischar(oneelement))
            oneelement(1,strfind(oneelement,'_')) = ' ';
            if (j==1)
                text_row = [ text_row oneelement];
            else
                text_row = [ text_row ' & ' oneelement];
            end
        end
        if (isnumeric(oneelement))
            if (j==1)
                text_row = [ text_row num2str(oneelement)];
            else
                text_row = [ text_row ' & ' num2str(oneelement)];
            end
        end
        if (j == ncols && i == nrows)
            text_row = [ text_row ' \n  '];
        end
        if (j == ncols && i ~= nrows)
            text_row = [ text_row ' \\\\ \n  '];
        end
    end
    fprintf(fid,text_row);
    text_mat = strvcat(text_mat,text_row); % text_mat
end
fprintf(fid,'\\end{tabular} \n');
fprintf(fid,'\\end{document} \n');
fclose(fid);
return
