function [arlag, res] = f_infovar(X,armax,method)
% f_infovar(X,armax,method) 
%    function provides var (var contains constant) lag length
%    for all considered models, the same effective sample size is used, 
%    following Ng and Perron (2005) OxBullEcStat
%                           
%    X (T,n)   vector of endogeneous variable
%    armax     maximum ar lag length
%    method    dummy string = 'bic' or 'aic' 
%
% Christian Schumacher, 7 March 2007
%

% get size
[T,n] = size(X); 

icmat = NaN(armax,1+2);
for i=1:armax,
    X_reg = X(armax-i+1:end,:);
    if (n > 1)
        varresult = vare(X_reg,i);
        neqs      = varresult(1).neqs;
        nlag      = i;
        resids    = horzcat(varresult.resid);
        nobs      = size(resids,1);
        sige      = (1/nobs)*resids'*resids; % no correction, see ltkepohl (1991), p. 132, 67
        aic       = log(det(sige)) + (nlag+1)*(neqs^2)*2/nobs;
        bic       = log(det(sige)) + (nlag+1)*(neqs^2)*log(nobs)/nobs;
    end
    if (n == 1)
        arresult = ar(X_reg,i);
        nlag     = i;
        resids   = arresult.y(i+1:size(arresult.y,1),:)-arresult.yhat;
        nobs     = size(resids,1);
        sige     = (1/nobs)*resids'*resids;
        aic      = log(det(sige)) + (nlag+1)*2/nobs; % see greene (2000), p. 306
        bic      = log(det(sige)) + (nlag+1)*log(nobs)/nobs;
    end
    icmat(i,:) = [ i aic bic ];
end

% sort output
aic_sort = sortrows(icmat,2); ar_aic=aic_sort(1,1); rhs_aic = aic_sort(1,2);
bic_sort = sortrows(icmat,3); ar_bic=bic_sort(1,1); rhs_bic = bic_sort(1,2);

% results
if (strcmp(method,'bic'))
    arlag = ar_bic; varresult = vare(X,arlag); res = horzcat(varresult.resid);
    return
end
if (strcmp(method,'aic'))
    arlag = ar_aic; varresult = vare(X,arlag); res = horzcat(varresult.resid);
    return
end

return
