function [deltamc,acc,ftmc,ftstarmc,nvarsmc,lammc,lamsparmc,phimc,phistarmc,psimc, ...
    sigetamc,sigetastarmc,er0mc,sigepsmc,taumc,betamc,rhomc,rhopostmc]=f_mcmc_sparfac(y,lam,phi, ...
    phistar,psim,sigeta,sigeps,tau,beta,rho,f_ind,p0,P0,q0,Q0,sigetanu0,sigetaS0,er0,sigeps0,tau0,beta0,rho0,...
    nvars,nobs,nf,psilag,philag,it,it0,rth,bf,prior,normp,utz,ftstar,lamspar,ft)

% model
% x_t= lam*f_t + xi_t
% phi(L)f_t = eta_t  eta_t ~ N(0,sigeta)
% psim(L)xi_t=e_t     e_t ~ N(0,sigeps)
%
% nvars: number of variables
% nobs: number of obs
% nf: number of factors
% psilag: autoregressive order of idiosyncratic component
% philag: autoregressive order of factor dynamics
% it= number of MCMC iterations
% it0: number of burn-in iterations
% rth: save every rth draw
% bf: flag to save posterior moments

%input:
% y        	data matrix: nobs x nvars, variables in columns
%initial values of
% lam		nvars x nf
% phi		nf x nf*philag
% psim       nvars x psilag
% sigeta    nf x nf
% sigeps    nvars x 1
% beta      nvars x nf
% rho       nf x 1

%output:
% mmc = fix(it-it0)/rth
% MCMC draws
% ftmc     factors  nobs x nf x mmc
% lammc    nvars x nf x mmc
% lamstarmc nvars x nf x mmc
% phimc    nf x nf*philag x mmc
% psimc    nvars x psilag x mmc
% sigetamc nf x nf x mmc
% sigepsmc nvars x 1 x mmc
% betamc   nvars x nf x mmc
% rhomc    nf x 1 x mmc

% define MCMC output
ind=[it0+1:rth:it];
MMC=length(ind);
mci=1;
ftnobs=nobs-psilag;

ftmc=zeros(ftnobs,nf,MMC);
ftstarmc=zeros(ftnobs,nf,MMC);
lammc=zeros(nvars,nf,MMC);
lamsparmc=zeros(nvars,nf,MMC);
nvarsmc=zeros(nf,MMC);
phimc=zeros(nf,nf*philag,MMC);
phistarmc=zeros(nf,nf*philag,MMC);
psimc=zeros(nvars,psilag,MMC);
sigetamc=zeros(nf,nf,MMC);
sigetastarmc=zeros(nf,nf,MMC);
sigepsmc=zeros(nvars,1,MMC);
taumc=zeros(nf,1,MMC);
rhomc=zeros(nf,1,MMC*(prior>0));
rhopostmc=zeros(nf,2,MMC*(prior>0));
betamc=zeros(nvars,nf,MMC*(prior==2));
er0mc=zeros(1,MMC*(prior>0));
deltamc=zeros(nvars,nf,MMC);
%lamspar=zeros(nvars,nf);

acc=0;
sigetast=sigeta;
disp(' ');
disp('sampling');
for mmc=1:it;
    
    if any(mmc==[500:500:it]);
        disp(['   ',num2str(mmc)]);
    end
    
    % condensed kalman filter, give data y,lam,sigeps,sigeta,phi; out:generated ft */
    ystar=y;
    if any(psilag);
        ystar=f_lagtrans_ind(y,psilag,psim); %transformed data for condensed state space
    end;
    ft=f_condkalfila(ystar,lam,phi,psim,sigeps,sigetast,psilag,philag,nf);

    % generating sigeta,theta
    [ac,phistar,psim,sigeps,sigeta]=f_simparstarexlamda(y,ft,lam,phi,phistar,psim,sigeps,sigetast,psilag,philag,f_ind,p0,P0,q0,Q0,sigetanu0,sigetaS0,sigeps0,er0,utz,normp);
    if it>it0;
        acc=acc+ac;
    end
    if prior
        [delta,ftstar,lamspar,tau,beta,rho,rhop,er0]=f_simparlamsparse(ystar,ft,lam,psim,sigeps,sigeta,tau,rho,psilag,tau0,beta0,rho0,prior,utz);
    elseif ~prior
        [ftstar,lamspar,lam,tau]=f_simparlamnormal(ystar,ft,lam,psim,sigeps,sigeta,tau,psilag,tau0,utz);
        delta=ones(size(lamspar));
    end

    % ensure that majority of entries in lam columns have positive sign
    flag_ensurepositivity='off';
    if strcmp(flag_ensurepositivity,'on')
        s_ind=f_checkpositivity(lamspar);
        for j=s_ind
            pid=find([1:nf]~=j);
            lamspar(:,j)=-lamspar(:,j);
            ft(:,j)=-ft(:,j);
            ftstar(:,j)=-ftstar(:,j);
            phistar(pid,[0:philag-1]*nf+j)=-phistar(pid,[0:philag-1]*nf+j);
            phistar(j,kron([0:philag-1]*nf,ones(1,length(pid)))+kron(ones(1,philag),pid))=-phistar(j,kron([0:philag-1]*nf,ones(1,length(pid)))+kron(ones(1,philag),pid));
        end
    end

    % random sign switch
    flag_randsignswitch='on';
    if strcmp(flag_randsignswitch,'on')
        index_sswitch=rand(1,nf)>0.5;
        mat_sswitch=ones(1,nf);
        mat_sswitch(index_sswitch)=mat_sswitch(index_sswitch)*(-1);
        lamspar=lamspar.*kron(mat_sswitch,ones(nvars,1));
        ft=ft.*kron(mat_sswitch,ones(ftnobs,1));
        ftstar=ftstar.*kron(mat_sswitch,ones(ftnobs,1));
        % lag polynomial
        perm_mat=diag(mat_sswitch); % permutation matrix
        phistar2=nan(size(phistar));
        for j=1:philag
            phistarj=phistar(:,(j-1)*nf+1:(j-1)*nf+nf); % poly at lag j
            phistar2(:,(j-1)*nf+1:(j-1)*nf+nf)=perm_mat*phistarj/perm_mat; % P*F(t)=(P*Phistar(j)*inv(P))*(P*F(t-1))+P*E(t)
        end
        phistar=phistar2; clear phistar2
    end

    % random column switch
    flag_randcolswitch='on';
    if strcmp(flag_randcolswitch,'on')
        index_cswitch=randperm(nf);
        lamspar=lamspar(:,index_cswitch);
        ftstar=ftstar(:,index_cswitch);
        % lag polynomial
        perm_mat=eye(nf);perm_mat=perm_mat(:,index_cswitch); % permutation matrix
        phistar2=nan(size(phistar));
        for j=1:philag
            phistarj=phistar(:,(j-1)*nf+1:(j-1)*nf+nf); % poly at lag j
            phistar2(:,(j-1)*nf+1:(j-1)*nf+nf)=perm_mat*phistarj/perm_mat; % P*F(t)=(P*Phistar(j)*inv(P))*(P*F(t-1))+P*E(t)
        end
        phistar=phistar2; clear phistar2
        % tau, rho
        tau=tau(index_cswitch);
        delta=delta(:,index_cswitch);
        if prior
            rho=rho(index_cswitch);
            rhop=rhop(index_cswitch,:);
            if prior==2;
                beta=beta(:,index_cswitch);
            end
        end
    end
    
    % transform lamspar to lam
    if normp
       [ve,va]=eig(lamspar'*lamspar);
       H=ve*sqrt(va)*ve';
       HI=ve*inv(sqrt(va))*ve';
       lam=lamspar*HI;
       sigetast=H*H';
       for j=1:philag;
           phi(:,(j-1)*nf+1:j*nf)= H*phistar(:,(j-1)*nf+1:j*nf)*HI;
       end
    elseif ~normp
       lam=lamspar;
       sigetast=sigeta;
       phi=phistar;
    end

    % saving
    if any(mmc==ind)
        deltamc(:,:,mci)=delta;
        ftmc(:,:,mci)=ft(end-ftnobs+1:end,:);
        ftstarmc(:,:,mci)=ftstar(end-ftnobs+1:end,:);
        lammc(:,:,mci)=lam;
        lamsparmc(:,:,mci)=lamspar;
        nvarsmc(:,mci)=sum(lamspar~=0,1)';
        if philag
            phimc(:,:,mci)=phi;
            phistarmc(:,:,mci)=phistar;
        end
        if psilag
            psimc(:,:,mci)=psim;
        end
        sigetamc(:,:,mci)=sigeta;
        sigetastarmc(:,:,mci)=sigetast;
        sigepsmc(:,:,mci)=sigeps;
        taumc(:,:,mci)=tau;
        if prior;
            rhomc(:,:,mci)=rho;
            rhopostmc(:,:,mci)=rhop;
            er0mc(1,mci)=er0;
            if prior==2;
                betamc(:,:,mci)=beta;
            end
        end
        mci=mci+1;
    end
end
acc=acc/(it-it0);

end
