function [tau0,beta0,rho0,p0,P0inf,q0,Q0inf,sigeps0,sigetanu0,sigetaS0,er0]=f_specify_prior(data,modparam)
% out: tau0 (nfx2), beta0=(a0,b0) rho0=(r0,s0) (both nfx2), p0 (nf^2*philagx1), P0inf ((nf^2*philag)x(nf^2*philag)),
% q0 (nvarsxpsilag) Q0inf(psilagxpsilagxnvars),
% sigetanu0=e0, sigetaS0=E0 (nfxnf)
% sigeps0=(u0,U0) (nvarsx2)

% get model specification
param_str_vec={'it','it0','rth','nf','s0','prior','philag','psilag',...
               'utz','f_ind','bf','const','m_adj','s_adj','normp'};
for i=1:size(param_str_vec,2)
    param_str=param_str_vec{1,i};
    eval([param_str,'=','modparam.',param_str,';']);
end
clear modparam

% prepare data
y_raw=data.data_fin;
y=f_standardise(y_raw);
nobs=size(y,1);
nvars=size(y,2);
N=nvars;

% prior specification p0,P0inf,q0,Q0inf
minn_var=0.09; %prior variance for the first autoregressive lag
minn_cov=0.03; %shrink factor for prior variance on lags of other variables (Minnesota type prior)
if ~f_ind
    prminn=f_minnesota(zeros(1,nf),zeros(1,nf),philag,minn_var,minn_cov)';
    P0inf=diag(prminn(:));
    p0=zeros(nf^2*philag,1);
elseif f_ind
    prminn=f_minnesota(zeros(1,1),zeros(1,1),philag,minn_var,minn_cov)';
    P0inf=diag(prminn(:));
    P0inf=P0inf(:,:,ones(1,nf));
    p0=zeros(nf,philag);
end
prminn=f_minnesota(zeros(1,1),zeros(1,1),psilag,minn_var,minn_cov)';
Q0inf=diag(prminn(:));
Q0inf=Q0inf(:,:,ones(1,nvars));
q0=zeros(nvars,psilag);

% loadings
sigeps0=kron(ones(nvars,1),[2,1]);
tau0=kron(ones(nf,1),[2,0.5]);
rho0=[];
beta0=[];
er0=0;
if prior==2;
    rho0=kron(ones(nf,1),[3.0,s0]);
    beta0=kron(ones(nf,1),[3.0,0.8]); %beta0_1=precision, beta0_2=mean, variance= beta0_2(1-beta0_2)/(1+beta0_1)
    er0=round(N*prod(1-rho0(:,2).*beta0(:,2),1)); %expected number of zero loadings row
elseif prior==1;
    rho0=kron(ones(nf,1),[3.0,s0]); %rho0_1=precision, rho0_2=mean
    er0=round(N*prod(1-rho0(:,2)));
end

% sig
sigetam=eye(nf)*(~normp + normp*(N-er0));
sigetanu0=(nf+2); %degrees of freedom for inverse wishart prior for sig
sigetaS0=sigetam*(sigetanu0-(nf+1)/2); %scale matrix of inv. wishart equal to E(W)=sig , mode is sig0/(nu0+(nvars+1)/2)

end
