function results=f_eval_relevance(data,modparam,lamdraws,fdraws,it_sel,folder)    

% nominal inclusion prob for inclusion/exclusion
clev=0.95;

% generate folder and for results
if not(exist(folder,'dir')==7)
    mkdir(folder);
end

% get model specification
disp(' ');
disp('model specification');
disp(modparam);
param_str_vec={'it','it0','rth','nf','s0','prior','philag','psilag',...
               'utz','f_ind','bf','const','m_adj','s_adj','normp'};
for i=1:size(param_str_vec,2)
    param_str=param_str_vec{1,i};
    eval([param_str,'=','modparam.',param_str,';']);
end
results.modparam=modparam;

% string representing key model specifications
modspec_string=['spec','_k',num2str(nf),'_prior',num2str(prior), ...
                '_s',num2str(s0*100,'%03.0f'),'_utz',num2str(utz)];

% diary
diary([folder,'diary_eval_relevance_',modspec_string,'.txt']);

% check
if size(lamdraws,3)~=size(fdraws,3)
    error('lamdraws and fdraws contain different numbers of draws')
end
it_raw=1:size(lamdraws,3);
if not(isempty(setdiff(it_sel,it_raw)))
    error('it_sel is not a subset of it_raw')
end
n_it=size(it_sel,2);
disp(' ');
disp(['number of draws ',num2str(n_it)]);
T=size(fdraws,1);
N=size(lamdraws,1);

% pick draws
lamdraws=lamdraws(:,:,it_sel);
fdraws=fdraws(:,:,it_sel);
lam_mean=mean(lamdraws,3);
lam_med=median(lamdraws,3);

% names
name_ser=data.name_ser;
name_ser_str=char(name_ser);
name_ser_str=name_ser_str(:,1:min(15,size(name_ser_str,2)));
name_ser2=cellstr(name_ser_str)';

% pval count posterior draws larger or smaller than zero (one-sided)
[relevant_hpd_index_single,relevant_hpd_index]=f_relevance_hpd(lamdraws,clev);
[relevant_mhpd_index_single,relevant_mhpd_index]=f_relevance_mhpd(lamdraws,clev,'mah');
[relevant_col_index_single,relevant_col_index]=f_relevance_col(lamdraws,clev);

% params for plots
maxsize=50; % max no series in heat plot
npic=ceil(N/maxsize);
maxsize=floor(N/npic);

% heat map lamdraws
flag_plot = 'on';
if (strcmp(flag_plot,'on'))
    load cmap
    figure;newplot;
    data_fig=lam_mean;
    maxdata=max(max(data_fig));
    mindata=min(min(data_fig));
    absmaxdata=max(abs(maxdata),abs(mindata));
    for i=1:npic
        index1=(i-1)*maxsize+1;
        index2=(i-1)*maxsize+maxsize;
        if i==npic
            index2=N;
        end
        subplot(1,npic,i);
        data_fig2=data_fig(index1:index2,:);
        data_fig2_new=data_fig2;
        data_fig2_new(1,1)=absmaxdata;
        data_fig2_new(end,end)=-absmaxdata;
        heatmap(data_fig2_new,[1:nf],lower(name_ser2(1,[index1:index2])),[],'Colormap',cmap,'Colorbar',true,'FontSize',2);
        hold on
        heatmap(data_fig2,[1:nf],lower(name_ser2(1,[index1:index2])),[]);
    end    
    % store figure
    fig_str=['fig_',modspec_string,'_heat_lammean'];
    set(gcf,'name',fig_str);
    save_fig_str=[folder,fig_str];
    saveas(gcf,save_fig_str);
end

% 0/1 heat map lamdraws based on univariate hpd
flag_plot = 'on';
if (strcmp(flag_plot,'on'))
    load cmap
    figure;newplot;
    for i=1:npic
        index1=(i-1)*maxsize+1;
        index2=(i-1)*maxsize+maxsize;
        if i==npic
            index2=N;
        end
        subplot(1,npic,i);
        heatmap(relevant_hpd_index([index1:index2],:),[],lower(name_ser2(1,[index1:index2])),[],'Colormap',flipud(gray(50)),'Colorbar',true,'FontSize',2);
    end    
    % store figure
    fig_str=['fig_',modspec_string,'_heat_lamhpdrelevant'];
    set(gcf,'name',fig_str);
    save_fig_str=[folder,fig_str];
    saveas(gcf,save_fig_str);
end

% hpd: in the groups, how many of the series are relevant (in %)?
digs=2;
relevant_index2=find(relevant_hpd_index);
avprob_hpd=mean(relevant_hpd_index);    
nosign_hpd=sum(sum(relevant_hpd_index_single))/(N*nf);
str_method='hpd';
avprob=avprob_hpd;nosign=nosign_hpd;
% local code below
group_names_unique=unique(lower(data.name_ser_group(1,:)));
n_groups=size(group_names_unique,2);
group_relevant=nan(n_groups,1);
n_group_str=cell(1,n_groups);
for i=1:n_groups
    index_group=strcmpi(group_names_unique{1,i},data.name_ser_group(1,:));
    index_group=find(index_group);
    n_vars_group=size(index_group,2);
    n_vars_incl_group=size(intersect(relevant_index2,index_group),1);
    group_relevant(i,1)=n_vars_incl_group/n_vars_group;
    str=[' (',num2str(n_vars_incl_group,'%10.0f'),' of ',num2str(n_vars_group, '%10.0f'),')'];
    n_group_str{1,i}=str;
end
disp(' '); disp([str_method,': How many series in groups have at least one loading different from zero (prob >= 0.95, ratio)']); disp(' ');
disp([char('group',char(group_names_unique)),blanks(size(char(group_names_unique),1)+1)',f_mat2str(group_relevant,1:size(group_relevant,2),digs), char(' ',char(n_group_str)) ]);
disp(' '); 
disp(['pct of variables with at least one significant loading: ',num2str(avprob*100.0)]); 
disp(['percent of lambdas different from zero (prob >= 0.95):  ',num2str(nosign*100.0)]); 
disp(' '); 
n_group_str=[f_mat2str(group_relevant,1:size(group_relevant,2),digs), char(' ',char(n_group_str))];
% local code above
n_group_str_hpd=n_group_str(2:end,:);

% mhpd: in the groups, how many of the series are relevant (in %)?
relevant_index2=find(relevant_mhpd_index);
avprob_mhpd=mean(relevant_mhpd_index);    
nosign_mhpd=sum(sum(relevant_mhpd_index_single))/(N*nf);
str_method='mhpd';
avprob=avprob_mhpd;nosign=nosign_mhpd;
% local code below
group_names_unique=unique(lower(data.name_ser_group(1,:)));
n_groups=size(group_names_unique,2);
group_relevant=nan(n_groups,1);
n_group_str=cell(1,n_groups);
for i=1:n_groups
    index_group=strcmpi(group_names_unique{1,i},data.name_ser_group(1,:));
    index_group=find(index_group);
    n_vars_group=size(index_group,2);
    n_vars_incl_group=size(intersect(relevant_index2,index_group),1);
    group_relevant(i,1)=n_vars_incl_group/n_vars_group;
    str=[' (',num2str(n_vars_incl_group,'%10.0f'),' of ',num2str(n_vars_group, '%10.0f'),')'];
    n_group_str{1,i}=str;
end
disp(' '); disp([str_method,': How many series in groups have at least one loading different from zero (prob >= 0.95, ratio)']); disp(' ');
disp([char('group',char(group_names_unique)),blanks(size(char(group_names_unique),1)+1)',f_mat2str(group_relevant,1:size(group_relevant,2),digs), char(' ',char(n_group_str)) ]);
disp(' '); 
disp(['pct of variables with at least one significant loading: ',num2str(avprob*100.0)]); 
disp(['percent of lambdas different from zero (prob >= 0.95):  ',num2str(nosign*100.0)]); 
disp(' '); 
n_group_str=[f_mat2str(group_relevant,1:size(group_relevant,2),digs), char(' ',char(n_group_str))];
% local code above
n_group_str_mhpd=n_group_str(2:end,:);

% col: in the groups, how many of the series are relevant (in %)?
relevant_index2=find(relevant_col_index);
avprob_col=mean(relevant_col_index);    
nosign_col=sum(sum(relevant_col_index_single))/(N*nf);
str_method='col';
avprob=avprob_col;nosign=nosign_col;
% local code below
group_names_unique=unique(lower(data.name_ser_group(1,:)));
n_groups=size(group_names_unique,2);
group_relevant=nan(n_groups,1);
n_group_str=cell(1,n_groups);
for i=1:n_groups
    index_group=strcmpi(group_names_unique{1,i},data.name_ser_group(1,:));
    index_group=find(index_group);
    n_vars_group=size(index_group,2);
    n_vars_incl_group=size(intersect(relevant_index2,index_group),1);
    group_relevant(i,1)=n_vars_incl_group/n_vars_group;
    str=[' (',num2str(n_vars_incl_group,'%10.0f'),' of ',num2str(n_vars_group, '%10.0f'),')'];
    n_group_str{1,i}=str;
end
disp(' '); disp([str_method,': How many series in groups have at least one loading different from zero (prob >= 0.95, ratio)']); disp(' ');
disp([char('group',char(group_names_unique)),blanks(size(char(group_names_unique),1)+1)',f_mat2str(group_relevant,1:size(group_relevant,2),digs), char(' ',char(n_group_str)) ]);
disp(' '); 
disp(['pct of variables with at least one significant loading: ',num2str(avprob*100.0)]); 
disp(['percent of lambdas different from zero (prob >= 0.95):  ',num2str(nosign*100.0)]); 
disp(' '); 
n_group_str=[f_mat2str(group_relevant,1:size(group_relevant,2),digs), char(' ',char(n_group_str))];
% local code above
n_group_str_col=n_group_str(2:end,:);

% fig: hist of all lam
nshow=5;
nfig=ceil(N/nshow);
for k=1:nfig
    index_var=(k-1)*nshow+1:(k-1)*nshow+nshow;
    if index_var(end)>N
        index_var=(k-1)*nshow+1:N;
    end
    index_var_str=[num2str(index_var(1)),'to',num2str(index_var(end))];
    jj=1;
    figure;newplot;
    for i=1:length(index_var)
        index_sel=index_var(i);
        for j=1:nf
            subplot(nshow,nf,jj);
            flag_dens='off';
            if strcmp(flag_dens,'on')
                [x,f]=dichte(squeeze(lamdraws(index_sel,j,:)));
                plot(x,f,'k-');
            else
                hist(squeeze(lamdraws(index_sel,j,:)),50);
            end
            set(gca,'FontSize',8);
            title(['f ',num2str(j),', var ',num2str(index_sel)],'FontSize',8);
            jj=jj+1;
        end
    end
    % store figure
    fig_str=['fig_',modspec_string,'_lamdrawshist_relevance','_var',index_var_str];
    set(gcf,'name',fig_str);
    save_fig_str=[folder,fig_str];
    saveas(gcf,save_fig_str);
end

% summary results table 
results.stat_labels={'utz';'s0';'k';'prior';'pct of sign lam';'pct of of rel vars'};
results.stat_hpd=[utz;s0;nf;prior;nosign_hpd;avprob_hpd];
results.stat_mhpd=[utz;s0;nf;prior;nosign_mhpd;avprob_mhpd];
results.stat_col=[utz;s0;nf;prior;nosign_col;avprob_col];

% group results table 
results.group_names_unique=group_names_unique;
results.n_group_str_hpd=n_group_str_hpd; % this is a char
results.n_group_str_mhpd=n_group_str_mhpd;
results.n_group_str_col=n_group_str_col;

diary off

end



function [relevant_hpd_index_single,relevant_hpd_index]=f_relevance_hpd(lamdraws,thresh)
N=size(lamdraws,1);nf=size(lamdraws,2);
pincl_hpd=nan(N,nf);
for i=1:N
    for j=1:nf
        lamij=squeeze(lamdraws(i,j,:));
        alpha=1-thresh;
        ki=f_hpd(lamij,alpha);
        pincl_hpd(i,j)=(min(ki)>0.0||max(ki)<0.0); 
    end
end
relevant_hpd_index_single=pincl_hpd;
relevant_hpd_index=sum(pincl_hpd,2)~=0.0; % 1 if at least one row hpd interval excludes zero (=relevant)
end

function ki=f_hpd(h,alpha)
%
% h draws anzahl der spalten = anzahl der variablen
% output:
% ki
%
h1=h;
h=h';
n = size(h,2);
k = size(h,1);
[ssort] = sort(h1);
ssort = ssort';
F = floor(n*alpha);
kiu = zeros(k,F+1);
kio = zeros(k,F+1);
kid = zeros(k,F+1);
ki = zeros(k,2);
for i = 0:F,
	kiu(:,i + 1) = ssort(:,i + 1);
	kio(:,i + 1) = ssort(:,(n -(F - i)));
	kid(:,i + 1) = kio(:,i+1) - kiu(:,i+1);
end
[~,I] = min(kid,[],2);
for i = 1:k,
   ki(i,:) = [kiu(i,I(i)) kio(i,I(i))];
end   
end

function [pincl_mhpd_index_single,pincl_mhpd_index]=f_relevance_mhpd(lamdraws,clev,besagmah_str)
% multivariate hpd
disp_flag='off';fig_flag='off';
N=size(lamdraws,1);nf=size(lamdraws,2);
vec_zero=zeros(1,nf);
inclprobdelta_besag_index_single=zeros(N,nf);
inclprobdelta_besag_index=zeros(N,1);
inclprobdelta_mah_index_single=zeros(N,nf);
inclprobdelta_mah_index=zeros(N,1);
for i=1:N
    if nf==1
        lami=squeeze(lamdraws(i,:,:));
    else
        lami=squeeze(lamdraws(i,:,:))';
    end
    [mhpd_besag,pval_besag]=f_mhpd_besag(lami,clev,disp_flag,vec_zero);
    ki_out=nan(nf,1);
    for j=1:nf
        ki=mhpd_besag(j,:);
        ki_out(j,:)=(min(ki)>vec_zero(j)||max(ki)<vec_zero(j)); 
    end
    inclprobdelta_besag_index_single(i,:)=ki_out';
    if sum(ki_out)>0
        % lami ci excludes zero in at least in one direction (==nf in all directions)
        inclprobdelta_besag_index(i,1)=1;
    end
    [zeroout_flag,pval_mah]=f_mhpd_eucmah(lami,clev,besagmah_str,disp_flag,fig_flag,vec_zero);
    inclprobdelta_mah_index_single(i,:)=nan;
    if zeroout_flag==1
        % lami ci excludes zero in at least in one direction (==nf in all directions)
        inclprobdelta_mah_index(i,1)=1;
    end
end
if strcmp(besagmah_str,'mah');
    pincl_mhpd_index_single=inclprobdelta_mah_index_single;
    pincl_mhpd_index=inclprobdelta_mah_index;
elseif strcmp(besagmah_str,'besag');
    pincl_mhpd_index_single=inclprobdelta_besag_index_single;
    pincl_mhpd_index=inclprobdelta_besag_index;
end

end

function [pincl_col_index_single,pincl_col_index]=f_relevance_col(lamdraws,clev)
% zero col based prob of irrelevance
disp_flag='off';fig_flag='off';
N=size(lamdraws,1);nf=size(lamdraws,2);
vec_zero=zeros(1,nf);
inclprobdelta_col_index_single=zeros(N,nf);
inclprobdelta_col_index=zeros(N,1);
for i=1:N
    if nf==1
        lami=squeeze(lamdraws(i,:,:));
    else
        lami=squeeze(lamdraws(i,:,:))';
    end
    avnozeros=sum((sum(lami==0,2))==nf,1)/size(lami,1); % lami (Nx2)
    inclprobdelta_col_index_single(i,:)=nan;
    if (1-avnozeros)>clev
        inclprobdelta_col_index(i,1)=1;
    end
    
end
pincl_col_index_single=inclprobdelta_col_index_single;
pincl_col_index=inclprobdelta_col_index;

end


