function results=f_simdatsparse(dgp_spec)

N=dgp_spec.N_sim;
T=dgp_spec.T_sim;
T0=dgp_spec.T0_sim;
nf=dgp_spec.nf_sim;
philag=dgp_spec.philag_sim;
psilag=dgp_spec.psilag_sim;
rho0_sim=dgp_spec.rho0_sim;

ftstar=zeros(T,nf);
ft=zeros(T,nf);
xt=zeros(T,N);
xi=zeros(T,N);

m0=[0.60 0.40]'; % mean of lam*
tausim0=kron(ones(nf,1),0.01); %variance of lam*

phistar=[0.3 0;0 0.8];
phi=zeros(size(phistar));

sigeps0=mean(1-m0(1:nf).^2);
beta0=kron(ones(nf,1),[30,0.8]); %beta0_1=precision, beta0_2=mean, variance= beta0_2(1-beta0_2)/(1+beta0_1)
tausim=tausim0(:,1);

% lamspar: step-by-step draw from 3-layer prior
lamspar=zeros(N,nf); 
lamspar_index=zeros(N,nf); 
for j=1:nf
    r0 = rho0_sim(j,1);
    s0 = rho0_sim(j,2);
    a0 = beta0(j,1);
    b0 = beta0(j,2);
    M = tausim(j);
    % layer 0: rho
    rho = betarnd(r0*s0,r0*(1-s0),1,N); 
    % layer 1: beta zero? No, then draw from beta distr
    index_unequalzero = rand(1,N)<=rho;
    beta = zeros(1,N);
    beta(1,index_unequalzero) = betarnd(a0*b0,a0*(1-b0),1,sum(index_unequalzero)); 
    % layer 2: lam* zero? No, then draw from normal
    index_unequalzero = rand(1,N)<=beta; N_unequalzero = sum(index_unequalzero);
    lamstar = zeros(1,N);
    lamstar(1,index_unequalzero) = m0(j)+sqrt(M).*randn(1,N_unequalzero);
    % ensure non-zero informative loading through truncated normal (non-informative loadings will introduced through zeros in dgp)
    lamstar_nonzero=nan(1,N_unequalzero);
    for i=1:N_unequalzero
        lamstar_nonzero(i) = f_rtnorm(0.1,inf,m0(j),sqrt(M));
    end
    % switch signs randomly 
    iswitch = rand(1,length(lamstar_nonzero))>0.5; lamstar_nonzero(iswitch) = lamstar_nonzero(iswitch)*(-1); % switch signs randomly
    lamstar(1,index_unequalzero) = lamstar_nonzero;
    % combine
    lamspar(:,j) = lamstar';
    lamspar_index(:,j) = index_unequalzero';  
end

% ensure more positive elements in lam* than negative elements
s_ind=f_checkpositivity(lamspar);
lamspar(:,s_ind)=-lamspar(:,s_ind);
lam=lamspar;

% H
H = eye(nf);
HI = inv(H);

% sigetastar, sigeps
sigetastar=eye(nf);
sigeps=1./gamrnd(kron(ones(N,1),[2]),1./kron(ones(N,1),[sigeps0]));

%phi, psi, phistar
minn_var=0.09; %prior variance for the first autoregressive lag
minn_cov=0.0278; %shrink factor for prior variance on lags of other variables (Minnesota type prior)

phistar=phistar(1:nf,1:nf);

%psi
if (psilag~=0)
    index=[];
    for j=1:psilag;
        index=[index [psilag+1-j:T+psilag-j]'];
    end
    prminn=minnesota(zeros(1,1),zeros(1,1),psilag,minn_var,minn_cov)';
    Q0ik=diag(prminn(:));
    Q0ik=Q0ik(:,:,ones(1,N));
    q0=zeros(N,psilag);
    psi=zeros(N,psilag);
    for j=1:N
        psi(j,:)= q0(j,:)'+chol(inv(Q0ik(:,:,j)))'*randn(psilag,1);
    end
    while any(abs(sum(psi,2))>1)
        ind=[find(abs(sum(psi,2))>1)];
        for j = ind';
            psi(j,:)= q0(j,:)'+chol(inv(squeeze(Q0ik(:,:,j))))'*randn(psilag,1);
        end
    end
end
% redefine for weak idio 
if (psilag==1)
    psi=(0.1).*ones(N,psilag);
end

%build system matrices for sparse-specification
Phi=[phistar;eye(nf*(philag-1),nf*philag)];
if (psilag~=0)
    Psi=zeros(N*psilag);
    for j=1:psilag
        Psi(1:N,(j-1)*N+1:j*N)= diag(psi(:,j));
    end
    Psi(N+1:end,:)=eye(N*(psilag-1),N*psilag);
end
Sigeps=sqrt(sigeps);

%initialize values and generate series
fti=zeros(nf*philag,1);   
if (psilag~=0)
    xit=[Sigeps.*randn(N,1);zeros(N*(psilag-1),1)];
else
    xit=Sigeps.*randn(N,1);
end    
etast=zeros(nf,T0+T);
for j=1:nf
    etast(j,:)=randn(1,T0+T);
end
etast= inv(chol(etast*etast'./(T0+T))')*etast;

for it=1:T0+T
    if (psilag~=0)
        xit=Psi*xit+[Sigeps.*randn(N,1);zeros(N*(psilag-1),1)];
    else
        xit=Sigeps.*randn(N,1);
    end
    fti=Phi*fti+[etast(:,it);zeros(nf*(philag-1),1)];
    xti=lamspar*fti(1:nf)+xit(1:N);
    if it>T0
        ii=it-T0;
        ftstar(ii,:)=(fti(1:nf))';
        ft(ii,:)=(H*fti(1:nf))';
        xi(ii,:)=xit(1:N)';
        xt(ii,:)=xti';
    end
end
for j=1:philag;
    phi(:,(j-1)*nf+1:j*nf)= H*phistar(:,(j-1)*nf+1:j*nf)*HI;
end

lamspar_true=lamspar;
lamspar_index_true=lamspar_index; 
lam_true=lam;
ft_true=ft;
ftstar_true=ftstar;
phistar_true=phistar;
if (psilag~=0)
    psi_true=psi;
else
    psi_true=[];
end
sigeps_true=sigeps;
m0out=m0;
Mout=M;

results=struct('xt',xt,'xi',xi,'ft_true',ft_true,'ftstar_true',ftstar_true, ...
    'lamspar_true',lamspar_true,'lamspar_index_true',lamspar_index_true,'lam_true',lam_true, ...
    'phistar_true',phistar_true,'psi_true',psi_true,'sigeps_true',sigeps_true,'m0out',m0out,'Mout',Mout);

end