function [lamstarmc_out,fstarmc_out,index_kmedoids_out,r2neg_index_mat,index_kmedoidsr2] = f_colchange_fbased(lamstarmc,fstarmc,r_fix)

[N,r,nrep]=size(lamstarmc);
T=size(fstarmc,1);
if r_fix~=r
    disp(['r ',num2str(r),', r_fix ',num2str(r_fix)]);
end

% fstarmc_stack=reshape(fstarmc,T,r*nrep);
legend_str_vec=cell(1,r);
for r_index=1:r
    legend_str_vec(1,r_index)={['row ',num2str(r_index)]};
end

lamstarmc_stack=reshape(lamstarmc,N,r*nrep);
fstarmc_stack=reshape(fstarmc,T,r*nrep);
clusteroptions = struct( ...
  'numTrials', 10, ...
  'useParallel', false, ...
  'displayTrialProgress', false, ...
  'normPower', 1000, ...
  'distanceMat', [] ...
  );

% kmedoids clustering based on absolute pearson correlation: removes col change
disp('kmedoids clustering based on distance defined as one minus absolute pearson correlation');
tstart = tic;
clustStruct=f_kmedoids(fstarmc_stack',r_fix,clusteroptions); % use lamstarmc_stack' if clustering shall be based on lam
                                                             % use r_fix clusters if more than r patterns are there
telapsed = toc(tstart);
disp(['time used by kmedoids ',num2str(telapsed/(60*60)),' hrs']);
index_kmedoids=clustStruct.clustInds;
index_kmedoidsr=reshape(clustStruct.clustInds',r,nrep); % index_kmedoidsr contains indices 1,...,r_fix to allocate pattern from 1,...,r

if r_fix==r
    % frhwirth-schnatter (2006): only posterior draws, where the resulting classification is 
    % a permutation of the original group indices, are considered
    sum_index_kmedoidsr=sum(index_kmedoidsr,1);% allocates each element in lamstarmc_stack to one of the r clusters, 
    sum_core=sum(1:r); % only permuted cols have a sum eq to sum(1:r)
    index_kmedoidsr_eq_sumcore=(sum_index_kmedoidsr==ones(1,nrep)*sum_core); 
    index_kmedoidsr2=index_kmedoidsr(:,index_kmedoidsr_eq_sumcore);
    nrep_clust=size(index_kmedoidsr2,2);
    % takes only values from clust_indices
    index_kmedoids2=kron(index_kmedoidsr_eq_sumcore,ones(1,r)); % vec index
    index_kmedoids3=index_kmedoids;
    index_kmedoids3(not(logical(index_kmedoids2)),1)=0;
    lamstarmc_clust=nan(N,r,nrep_clust);
    fstarmc_clust=nan(T,r,nrep_clust);
    for i=1:r
        lamstarmc_clust(:,i,:)=lamstarmc_stack(:,find(index_kmedoids3==i));
        fstarmc_clust(:,i,:)=fstarmc_stack(:,find(index_kmedoids3==i));
    end
    r_new=r;
end

if r_fix>r
    index_kmedoidsr_permc=[]; % search those indices where at least r_fix-r permutations of [1:r_fix] can be found (else means )
    for i=1:nrep
        if length(intersect(1:r_fix,index_kmedoidsr(:,i)))>=r
            index_kmedoidsr_permc=[index_kmedoidsr_permc,i]; %#ok<AGROW>
        end
    end
    nrep_clust1=length(index_kmedoidsr_permc);
    lamstarmc_clust=nan(N,r_fix,nrep_clust1);
    fstarmc_clust=nan(T,r_fix,nrep_clust1);
    for i=1:nrep_clust1
        for j=1:r_fix
            index_temp=index_kmedoidsr(:,index_kmedoidsr_permc(i))==j;
            if not(all(index_temp==0))
                lamstarmc_clust(:,j,i)=lamstarmc(:,index_kmedoidsr(:,index_kmedoidsr_permc(i))==j,index_kmedoidsr_permc(i));
                fstarmc_clust(:,j,i)=fstarmc(:,index_kmedoidsr(:,index_kmedoidsr_permc(i))==j,index_kmedoidsr_permc(i));
            end
        end
    end
    disp(['nrep_clust1 ',num2str(nrep_clust1)]);
    if r_fix>1
        nanlam=sum(isnan(squeeze(lamstarmc_clust(1,:,:))),2);
        nanf=sum(isnan(squeeze(fstarmc_clust(1,:,:))),2);
    else
        nanlam=sum(isnan(squeeze(lamstarmc_clust(1,:,:))),1);
        nanf=sum(isnan(squeeze(fstarmc_clust(1,:,:))),1);
    end        
    disp(['nanlam ',num2str(nanlam')]);
    disp(['nanf   ',num2str(nanf')]);
    addcut_flag='cut';
    if strcmp(addcut_flag,'cut')
        % keep only those r patterns with the least nans
        [~,leastnan_index]=sort(nanf,'ascend');
        leastnan_index(1:r)
        lamstarmc_clust=lamstarmc_clust(:,leastnan_index(1:r),:);
        fstarmc_clust=fstarmc_clust(:,leastnan_index(1:r),:);
        r_new=r;
    elseif strcmp(addcut_flag,'add')
        % keep all r_fix patterns
        r_new=r_fix;
    end
    if r_new>1
        nanlam=sum(isnan(squeeze(lamstarmc_clust(1,:,:))),2);
        nanf=sum(isnan(squeeze(fstarmc_clust(1,:,:))),2);
    else
        nanlam=sum(isnan(squeeze(lamstarmc_clust(1,:,:))),1);
        nanf=sum(isnan(squeeze(fstarmc_clust(1,:,:))),1);
    end        
    disp(['nanlam ',num2str(nanlam')]);
    disp(['nanf   ',num2str(nanf')]);
    % harmonize number of replications
    nrep_clust=nrep_clust1-max(nanf);
    lamstarmc_clust2=nan(N,r_new,nrep_clust);
    fstarmc_clust2=nan(T,r_new,nrep_clust);
    for i=1:r_new
        lamstarmc_clust_cut=lamstarmc_clust(:,i,not(isnan(squeeze(lamstarmc_clust(1,i,:)))));
        lamstarmc_clust2(:,i,:)=lamstarmc_clust_cut(:,:,end-nrep_clust+1:end);
        fstarmc_clust_cut=fstarmc_clust(:,i,not(isnan(squeeze(fstarmc_clust(1,i,:)))));
        fstarmc_clust2(:,i,:)=fstarmc_clust_cut(:,:,end-nrep_clust+1:end);
    end
    lamstarmc_clust=lamstarmc_clust2;
    fstarmc_clust=fstarmc_clust2;
    clear lamstarmc_clust2 fstarmc_clust2
    index_kmedoidsr2=[];index_kmedoids3=[];
end

% remove sign change in each col/cluster
lamstarmc_clust_sign=lamstarmc_clust;
fstarmc_clust_sign=fstarmc_clust;
r2neg_index_mat=nan(r_new,nrep_clust);
for i=1:r_new
    lamstarmc_clust_i=squeeze(lamstarmc_clust(:,i,:));
    r2=corrcoef([lamstarmc_clust_i(:,end),lamstarmc_clust_i(:,1:end-1)]);
    r2=r2(1,2:end)';
    r2neg_index=logical(r2<0); % [r2mat,r2matneg_index,r2matpos,logical(r2matpos<0)]
    r2neg_index=logical([r2neg_index;zeros(1,1)]); % note: r2 computed relative to last rep!
    r2neg_index_mat(i,1:size(r2neg_index,1))=r2neg_index;
    lamstarmc_clust_i(:,r2neg_index)=lamstarmc_clust_i(:,r2neg_index)*-1.0;
    lamstarmc_clust_sign(:,i,:)=lamstarmc_clust_i;
    fstarmc_clust_i=squeeze(fstarmc_clust(:,i,:));
    fstarmc_clust_i(:,r2neg_index)=fstarmc_clust_i(:,r2neg_index)*-1.0;
    fstarmc_clust_sign(:,i,:)=fstarmc_clust_i;
end

lamstarmc_out=lamstarmc_clust_sign;
fstarmc_out=fstarmc_clust_sign;
index_kmedoids_out=index_kmedoids3;

end
